;+
; NAME:     CANECDIP.PRO
;
; PURPOSE:  Calls CANECDIP.EXE and returns the EDFL COORDINATES
;              or the reverse
;
;  INPUT: FLAG           transformation direction flag
;             =0         transform from GEOCentric to ECcentric DIpole
;             =1         transform from ECcentric DIpole to GEOCentric
;
;         GCLAT          array of GeoCentric Lats (rads) of points
;         GCLON          array of GeoCentric Lons (rads) of points
;         GCRKM          array of GeoCentric distances ( km) of the points 
;
;  OUTPUT: ELAT          array ECcentric DIpole latitudes ( radians),
;          ELON          array ECcentric DIpole longitudes (radians),
;          ERKM          array ECcentric DIpole distances  (km).
;
;  local variables
;          NO             the number of points to be transformed
;          BMAGR         topocentric radial comp of B at trace point
;          BMAGT         topocentric south  comp of B at trace point
;          BMAGP         topocentric east   comp of B at trace point
;          BMAGB         Magnitude of B               at trace point
;
; NOTES
;        for FLAG=1    ELAT,ELON determine the starting point
;                      and field line tracing is done to the geocentric
;                      distance specified by GCRKM (a needed input).
;                      In this case GCLAT and GCLON are outputs.
;
;                      If no solution exists routine outputs LAT=-99.9999
;                                                            LON=999.9999
;                      No solution exists when the field line specified by
;                      the input parameters does not reach to the specified
;                      geocentric distance.
;
;                      A problem occurrs because it is not possible to
;                      know in advance exactly where to stop. Therefore
;                      it is usual to call CANECDIP twice; first with an
;                      approximate GCRKM value and then a second time
;                      with GCRKM adjusted by the amount appropriate to
;                      reach the desired stopping point - see routine
;                      used in VIKGRID_COORDS_ECDTOGC.FOR 
;;
; MODIFICATION HISTORY:
;     First written by D. D. Wallis Nov 1990 
;     upgraded to IDL version 2 - J. Trickey - Apr 1991
;     pointer to CANECDIP.EXE changed TJH 3-FEB-1993 
;-

PRO CANECDIP,FLAG,GCLAT,GCLON,GCRKM,ELAT,ELON,ERKM

;
;      +-------------------------------------------------------------------+
;      |Check Number of Parameters.                                        |
;      +-------------------------------------------------------------------+
;
if n_params(0) ne 7 then stop, $
   'CANECDIP.PRO error: You must provide 7 parameters'
;
;      +-------------------------------------------------------------------+
;      |Check Input ARRAYS        .                                        |
;      +-------------------------------------------------------------------+
   if (Flag eq 0) then no=n_elements(gclat)
   if (Flag eq 0) then nl=n_elements(gclon)
   if (Flag eq 0) then nr=n_elements(gcrkm)
   if (Flag eq 1) then no=n_elements(elat)
   if (flag eq 1) then nl=n_elements(elon)
   if (flag eq 1) then nr=n_elements(erkm)
   if (flag eq 1) then nk=n_elements(gcrkm)
   if ((no ne nl) or (no ne nr)) then stop,'CANECDIP: In array lengths not =' 
   if (flag eq 1) then begin 
     if (no ne nk) then stop,'CANECDIP: In array Lengths not =' 
   endif

;      +-------------------------------------------------------------------+
;      |Define the logical needed to call CANECDIP                         |
;      +-------------------------------------------------------------------+
if trnlog('Canecdip_exe',logname) ne 1 then begin     
   setlog,'Canecdip_exe', 'utl$idl$dan:Canecdip.EXE' 
endif

;      +-------------------------------------------------------------------+
;      |                                                                   |
;      |   Call CANECDIP.EXE. We define the calling parameters here.       |
;      |                                                                   |
;      +-------------------------------------------------------------------+
bmr=0.0 
bmp=0.0 
bmt=0.0 
bmb=0.0
no=long(no)
if (flag eq 0) then flag=0l
if (flag eq 1) then flag=1l
if (flag eq 0) then begin 
   elat=fltarr(no) 
   elon=elat 
   erkm=elat 
   ela=0.0 
   elo=0.0 
   erk=0.0 
endif
if (flag eq 1) then begin 
   gcla=0.0 
   gclo=0.0 
   gclat=fltarr(no) 
   gclon=gclat 
endif
for i=0,no-1 do begin                           
   if (flag eq 0) then begin                       
      gcla=gclat(i) 
      gclo=gclon(i)               
      gcrk=gcrkm(i) 
   endif                       
   if ( flag eq 1) then begin                      
      ela=elat(i) 
      elo=elon(i)                   
      erk=erkm(i) 
      gcrk=gcrkm(i) 
   endif         
   print,'CANECDIP: ',gcla*!radeg,gclo*!radeg,gcrk
   istat=call_vms('CANECDIP_EXE','Canecdip',no,FLAG,gcla,gclo,gcrk,$
                        ela,elo,erk,bmr,bmt,bmp,bmb)
   if (flag eq 0) then begin 
       elat(i)=ela          
       elon(i)=elo  
       erkm(i)=erk  
   endif        
   if (flag eq 1) then begin
       gclat(i)=gcla        
       gclon(i)=gclo  
       gcrkm(i)=gcrk 
   endif     
endfor
return
end
