;+
; Name: FIXGAPS.PRO
; Purpose: Linearly interpolate across gaps tagged with given value.
; Category: Data cleaning.
; Calling sequence: FIXGAPS,A[,TAGG[,EFLAGG]]
; Inputs: A = data array to fix.
;         TAGG = tag value.  Gaps are filled with this value or greater
;                Optional - if absent, TAGG is set to 99990.
;	       EFLAGG = optional flag, if non-zero then fix gaps at end by
;           replicating last good value. Default is fix end gaps.
; Outputs: A = fixed array.
; Procedure: uses LINFILL
; Modification history: Ray Sterner,   6 Mar, 1985.
;	Johns Hopkins University Applied Physics Laboratory.
;	RES, 16 Dec, 1987 --- added eflag.
;  27-MAY-1991 15:14:21.21 TJH/NRCC. Modified to degap all values >= TAGG.
;           and to fill values at beginning of array with first good point.
;           If no good values are found, just return with a warning message.
;           If EFLAGG is not given, defaults to fix end gaps.
;-

	PRO FIXGAPS,A,TAGG,EFLAGG

	IF N_PARAMS(0) eq 0 THEN BEGIN
	  PRINT,'FIXGAPS, A [, TAGG[, EFLAGG]]
	  PRINT,'  Linearly interpolate across data gaps in array A
	  PRINT,'  flagged by value >= TAGG. If EFLAGG is not given or is non-zero
	  PRINT,'  then fix any gap at end of data by replicating last good
	  PRINT,'  value. If TAGG not given, set it 99990.'
	  RETURN
	ENDIF

;      +-------------------------------------------------------------------+
;      |27-MAY-1991 15:32:44.95 TJH. Modified so that EFLAG defaults to 1  |
;      +-------------------------------------------------------------------+
	IF N_PARAMS(0) LT 3 THEN EFLAG = 1 else eflag=eflagg

;      +-------------------------------------------------------------------+
;      |added by Hughes 27-MAY-1991 13:59:50.46. If TAG not supplied,      |
;      |assume it is 99990. Also recode so that all values >= TAG are      |
;      |fixed.                                                             |
;      +-------------------------------------------------------------------+
   if n_params(0) eq 1 then tag=99990. else tag=tagg

   ibad=where(a ge tag)
;      +-------------------------------------------------------------------+
;      |27-MAY-1991 15:12:12.97 TJH. Do nothing if all values of A >= TAG  |
;      +-------------------------------------------------------------------+
   if n_elements(ibad) eq n_elements(a) then begin
      print,'Warning: no good values found. Nothing done to data'
      return
   endif

	S = SIZE(A)
	NR = S(S(0))

;      +-------------------------------------------------------------------+
;      |27-MAY-1991 15:09:59.57. TJH. Check start of data for fills. If    |
;      |found, fill them with value of first good point.                   |
;      +-------------------------------------------------------------------+
   igood=where(a lt tag)
   if igood(0) ne 0 then begin
      a(0:igood(0)-1)=a(igood(0))
      print,'Warning: First ',strtrim(string(igood(0)),2), ' data filled with first good value'
   endif

;      +-------------------------------------------------------------------+
;      |27-MAY-1991 15:10:41.97. TJH. Start at first good point instead    |
;      |of 0                                                               |
;      +-------------------------------------------------------------------+
	FOR I = igood(0), NR-1 DO BEGIN	; search for start of a gap.

	  IF A(I) GE TAG THEN BEGIN	; found one.
	    I1 = I - 1   		; last good value before gap.

	    FOR J = I+1, NR-1 DO BEGIN	; search for end of gap.
	      IF A(J) lt TAG THEN BEGIN ; found it.
	        I2 = J			; first good value after gap.
	        GOTO, FIX		; go fix gap.
	      ENDIF
	    ENDFOR ; J.

	    IF EFLAG EQ 0 THEN BEGIN
	      PRINT,'Error: gap to end of data. Not fixed.'
	      GOTO, DONE
	    ENDIF ELSE BEGIN
	      T = A(I1)			; Last good value.
	      FOR J = I1, NR-1 DO A(J) = T
	      PRINT,'Warning: gap to end of data filled with last good value.'
	      GOTO, DONE
	    ENDELSE

FIX:	    LINFILL, A, I1, I2
	    I = I2
	  ENDIF

	ENDFOR  ; I.

DONE:	RETURN

	END
