PRO oplot_sphere,lat_data,long_data,drop=drp,$
    altitude=alt,radius=rad,symsize=smsz,psym=psm,$
    linestyle=lnstl,color=clr,thick=thk,$
    continent=cont,edfl=dipole        
;+
; NAME:
;    OPLOT_SPHERE
; PURPOSE:
;    To plot lines or points on a sphere or globe. The sphere must first be 
;    gererated using PLOT_SPHERE.
; CATEGORY:
;    plotting
; CALLING SEQUENCE:
;    OPLOT_SPHERE,LAT_DATA,LONG_DATA
;
;    OPLOT_SPHERE,LAT_DATA,LONG_DATA [,DROP=value]
;          [,ALTITUDE=array[,RADIUS=value]]  [,PSYM=symbol] [,SYMSIZE=value]
;          [,LINESTYLE=style] [,COLOR=color] [,THICK=thick]
;          [,/CONTINENT [,/EDFL]]
; INPUTS:
;    LAT_DATA  real array  - latitude values of the points to be plotted
;                            on the sphere 
;    LONG_DATA real array  - longitude values of the points to be plotted
;                            on the sphere
;
; OPTIONAL INPUT PARAMETERS: 
;    None
; KEYWORD PARAMETERS:
;    DROP - Used to set data dropout value. default = 99999.0
;    ALTITUDE - Array containing the relative altitude above the surface
;               of the sphere. These values should be scaled relative to
;               the earth radius.
;    RADIUS - Used to change the relative value of the earth radius.
;             Default=1.
;    PSYM   - determines symbol type to be used for these points
;    SYMSIZE - controls the size of the symbols plotted, default = 1.0
;    LINESTYLE - determines the line style used to plot data
;    COLOR - determines the color used to plot data
;    CONTINENT  - If set, the continents and major islands of the world
;                 will be drawn on the sphere
;    EDFL       - If set, the continents will be drawn in EDFL coordinates,
;                 CONTINENT must also be specified.
; OUTPUTS:
;    none
; OPTIONAL OUTPUT PARAMETERS:
;    None
; COMMON BLOCKS:
;    PLOT_SPHR,LATV,LONGV
;      LATV  real value  - latitude coresponding to the center of the
;                          view of the sphere
;      LONGV real value  - longitude coresponding to the center of the
;                          view of the sphere
; SIDE EFFECTS: None
; RESTRICTIONS: None
; PROCEDURE:
;    The sphere must first be defined by a call to PLOT_SPHERE.
;    Ex. Plot_sphere,lat,long
; MODIFICATION HISTORY:
;    Created in IDL 2 - Jan. 1991 - J.D. Trickey
;      added COLOR    - April 1991 - J. D. Trickey
;    CONINENT added   - May 1991 - J. D. Trickey
;-
;
common plot_sphr,lat_val,long_val
;
npts=n_elements(lat_data)
;
if n_elements(drp) ne 0 then drop_val=drp else drop_val=99999.0
;
If keyword_set(smsz) then size=smsz else size=1.0
If n_elements(psm) ne 0 then begin
    psym_old=!p.psym
    !p.psym = psm
endif
If n_elements(thk) ne 0 then begin
    thick_old=!p.thick
    !p.thick = thk
endif
If n_elements(lnstl) ne 0 then begin
    linestyle_old=!p.linestyle
    !p.linestyle = lnstl
endif
If n_elements(clr) ne 0 then begin
    color_old=!p.color
    !p.color = clr
endif
;
if keyword_set(rad) then earth_rad=rad else earth_rad=1.
;
;
; get map if desired
   if keyword_set(cont) then begin
   	GET_LUN,LUN
      if keyword_set(dipole) then begin
   	   OPENR,LUN,'DAN$:[WALLIS.CANOP.UTIL.IDL]CONTINENT_EDFL.DAT',/variable
      endif else begin
    	   OPENR,LUN,'IDL$SYSPROG:CONTINENT.DAT/UNFORMATTED'
      endelse
   endif
;
;
thet=(90-lat_val)/!radeg 
phet=long_val/!radeg
rot1=[[cos(phet),sin(phet),0],[-sin(phet),cos(phet),0],[0,0,1.0]]
rot2=[[cos(thet),0,-sin(thet)],[0,1.0,0],[sin(thet),0,cos(thet)]]
if n_elements(alt) ne 0 then begin
    diameter=alt+earth_rad
endif else diameter=fltarr(npts)+earth_rad
;
    lon=long_data
    nx=fltarr(npts)
    ny=fltarr(npts)
    nz=fltarr(npts)
    p=where(lon gt 180.0) 
    if(p(0) ne -1) then lon(p)=lon(p)-360.0 
    p=where(lon lt -180.) 
    if(p(0) ne -1) then lon(p)=lon(p)+360.0
    for i=0,npts-1 do begin 
      the=lat_data(i)  
      phi=lon(i)
      z=diameter(i)             
      if ((abs(the) le 90.)and (abs(phi) le 360.)) then begin 
          th=(90-the)/!radeg 
          ph=lon(i)/!radeg            
          pos=[sin(th)*cos(ph),sin(th)*sin(ph),cos(th)]*z
          pn=pos#rot1#rot2                                 
          nx(i)=-pn(0) 
          ny(i)=pn(1) 
          nz(i)=pn(2)         
      endif else begin                                     
          nx(i)=-10  
          ny(i)=-10 
          nz(i)=-1 
      endelse                                              
    endfor
    nr=fltarr(npts)
    for i=1,npts-1 do nr(i)=sqrt(nx(i)*nx(i)+ny(i)*ny(i))
    T=where(nz lt 0.0 and nr lt 1.0)
    if (t(0) ne -1) then begin 
       nx(t)=drop_val
       ny(t)=drop_val
    endif
    T=where(nx lt drop_val)
    if (t(0) ne -1) then oPLOT_drop,ny,nx,drop=drop_val,symsize=size
;
npts=0  ; a 2-byte integer
if keyword_set(cont) then begin
   WHILE NOT EOF(LUN) DO BEGIN
    readu,LUN,NPTS,MINMAX & NPTS=NPTS/2
    LON=FLTARR(NPTS) & LAT=FLTARR(NPTS) & readu,LUN,LAT,LON
    nx=fltarr(npts) & ny=nx & nz=nx 
    p=where(lon gt 180.0) & if(p(0) ne -1) then lon(p)=lon(p)-360.0 
    p=where(lon lt -180.) & if(p(0) ne -1) then lon(p)=lon(p)+360.0
    for i=0,npts-1 do begin & the=lat(i)  & phi=lon(i)             
      if ((abs(the) le 90.)and (abs(phi) le 360.)) then begin 
          th=(90-the)/!radeg & ph=lon(i)/!radeg            
          pos=[sin(th)*cos(ph),sin(th)*sin(ph),cos(th)]    
          pn=pos#rot1#rot2                                 
          nx(i)=-pn(0) & ny(i)=pn(1) & nz(i)=pn(2)         
      endif else begin                                     
          nx(i)=-10 & ny(i)=-10 & nz(i)=-1 
      endelse                                              
    endfor
    T=where(nz ge 0.0)
    if (t(0) ge 0) then oPLOT,Ny(T),nx(T)	   
   ENDWHILE
  	FREE_LUN,LUN			  
endif
;
;
If n_elements(psm) ne 0 then begin
    !p.psym = psym_old
endif
If n_elements(thk) ne 0 then begin
    !p.thick = thick_old
endif
If n_elements(lnstl) ne 0 then begin
    !p.linestyle = linestyle_old
endif
If n_elements(clr) ne 0 then begin
    !p.color = color_old
endif
;
;
RETURN
END
