PRO polyfill_sphere,lat_data,long_data,$
    altitude=alt,radius=rad,symsize=smsz,psym=psm,$
    linestyle=lnstl,thick=thk,fill_pattern=filpat,$
    line_fill=linfil,spacing=spac,orientation=ortn
;+
; NAME:
;    POLYFILL_SPHERE
; PURPOSE:
;    To fill the interior of a region on a sphere or globe, inclosed by
;    a 2-D polygon. The sphere must first be generated using PLOT_SPHERE.
; CATEGORY:
;    plotting
; CALLING SEQUENCE:
;    POLYFILL_SPHERE,LAT_DATA,LONG_DATA
;
;    POLYFILL_SPHERE,LAT_DATA,LONG_DATA  [,ALTITUDE=array] [,RADIUS=value] 
;          [,PSYM=symbol] [,SYMSIZE=value] 
;          [,/LINE_FILL] [,LINESTYLE=style]
;          [,THICK=thickness] [,SPACING=space] [,ORIENTATION=angle]
;          [,FILL_PATTERN=fill_pattern] 
; INPUTS:
;    LAT_DATA  real array  - latitude values of the points to be connected
;                            on the sphere 
;    LONG_DATA real array  - longitude values of the points to be connected
;                            on the sphere
;
; OPTIONAL INPUT PARAMETERS: 
;    None
; KEYWORD PARAMETERS:
;    ALTITUDE - Array containing the relative altitude above the surface
;               of the sphere. These values should be scaled relative to
;               the earth radius. (ex 1.0 = 1 earth radius above surface)
;    RADIUS - Used to change the relative value of the earth radius.
;             Default=1.
;    PSYM   - determines symbol type to be used for these points
;    SYMSIZE - controls the size of the symbols plotted, default = 1.0
;    LINE_FILL - indicates the the area of the polygon is to be filled
;                with lines
;    LINESTYLE - determines the line style used to fill the polygon
;    THICK - determines the thickness of the lines used to fill the polygon
;    SPACING - determines the spacing of the lines used to fill the polygon
;    ORIENTATION - determines the angle of the lines used to fill the polygon
;                  relative to the horizontal
;    FILL_PATTERN - determines the hardware dependant fill pattern index.
; OUTPUTS:
;    none
; OPTIONAL OUTPUT PARAMETERS:
;    None
; COMMON BLOCKS:
;    none
; SIDE EFFECTS: None
; RESTRICTIONS: None
; PROCEDURE:
;    The sphere must first be defined by a call to PLOT_SPHERE.
;    Ex. Plot_sphere,lat,long
; MODIFICATION HISTORY:
;    Created in IDL 2 - Feb. 1991 - J.D. Trickey
;-
;
common plot_sphr,lat_val,long_val
;
npts=n_elements(lat_data)
;
If keyword_set(smsz) then size=smsz else size=1.0
If keyword_set(filpat) then filpat_val=filpat else filpat_val=0
If keyword_set(linfil) then linfil_val=linfil else linfil_val=0
If keyword_set(spac) then spac_val=spac else spac_val=0
If keyword_set(ortn) then ortn_val=ortn else ortn_val=0
If n_elements(psm) ne 0 then begin
    psym_old=!p.psym
    !p.psym = psm
endif
If n_elements(lnstl) ne 0 then begin
    linestyle_old=!p.linestyle
    !p.linestyle = lnstl
endif
If n_elements(thk) ne 0 then begin
    thick_old=!p.thick
    !p.thick = thk
endif
;
if keyword_set(rad) then earth_rad=rad else earth_rad=1.
;
thet=(90-lat_val)/!radeg 
phet=long_val/!radeg
rot1=[[cos(phet),sin(phet),0],[-sin(phet),cos(phet),0],[0,0,1.0]]
rot2=[[cos(thet),0,-sin(thet)],[0,1.0,0],[sin(thet),0,cos(thet)]]
if n_elements(alt) ne 0 then begin
    diameter=alt+earth_rad
endif else diameter=fltarr(npts)+earth_rad
;
    lon=long_data
    nx=fltarr(npts)
    ny=fltarr(npts)
    nz=fltarr(npts)
    p=where(lon gt 180.0) 
    if(p(0) ne -1) then lon(p)=lon(p)-360.0 
    p=where(lon lt -180.) 
    if(p(0) ne -1) then lon(p)=lon(p)+360.0
    for i=0,npts-1 do begin 
      the=lat_data(i)  
      phi=lon(i)
      z=diameter(i)             
      if ((abs(the) le 90.)and (abs(phi) le 360.)) then begin 
          th=(90-the)/!radeg 
          ph=lon(i)/!radeg            
          pos=[sin(th)*cos(ph),sin(th)*sin(ph),cos(th)]*z
          pn=pos#rot1#rot2                                 
          nx(i)=-pn(0) 
          ny(i)=pn(1) 
          nz(i)=pn(2)         
      endif else begin                                     
          nx(i)=-10  
          ny(i)=-10 
          nz(i)=-1 
      endelse                                              
    endfor
    nr=fltarr(npts)
    for i=1,npts-1 do nr(i)=sqrt(nx(i)*nx(i)+ny(i)*ny(i))
    T=where(nz ge 0.0 or nr ge 1.0)
;
    if (t(0) ne -1) then $
            polyfill,ny(t),nx(t),spacing=spac_val,$
            fill_pattern=filpat_val,$
            line_fill=linfil_val,symsize=size,orientation=ortn_val
;
If n_elements(psm) ne 0 then begin
    !p.psym = psym_old
endif
If n_elements(lnstl) ne 0 then begin
    !p.linestyle = linestyle_old
endif
If n_elements(thk) ne 0 then begin
    !p.thick = thick_old
endif
;
RETURN
END
