;+
; NAME:
;       CADI_KVEC2VEL
; PURPOSE:
;       Fit velocities to CADI KVEC (source k-vector) data.
; CALLING SEQUENCE:
;       VEL = CADI_KVEC2VEL( CADI_KVEC )
; INPUTS:
;       CADI_KVEC: Variable of CADI structure containing KVEC data.
; KEYWORD PARAMETERS:
;       MISSING_VEL: Scalar velocity structure, returned with the value of a
;          'missing' velocity.
; OUTPUTS:
;       VEL = Vector of velocity structures, one element for each time
;          represented in CADI_KVEC.
;       The fields of the velocity structure are:
;          TIME:      CANOPUS time.
;          NSOURCES:  Number of sources in fit.
;          FXRANGE:   2-element range of indices of sounding freqs of sources.
;          HXRANGE:   2-element range of indices of virtual ranges of sources.
;          AZI:       Azimuth of horizontal component of velocity, degrees.
;          HOR:       Magnitude of horizontal component of velocity, m/s.

;          HOREW
;          HORNS
;          HORSUN
;          HORDUSK
;          VER:       Vertical component of velocity, m/s.
;          AZI_ERR:   Error of azimuth, degrees.
;          HOR_ERR:   Error of horizontal speed, m/s.
;          VER_ERR:   Error of vertical velocity, m/s.
;          RMS_ERROR: RMS diff'ce between obs'd and modelled Doppler shift, Hz.
;       Azimuth is from geographic north: North=0, east=90.
;       Errors are standard error (one standard deviation) from the SVD fit.
; PROCEDURE:
;       For each time represented in CADI_KVEC, find the set of sources
;       corresponding to that time and fit a velocity to it with CADI_FIT_VEL.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 18 Oct 1994: Written.
;       GRANT, 8 Feb 1995: Replaced some code with CADI_TBLOCKS call.
;       JWM, 9 Oct 2002, added velocity antisunward and dawn dusk
;-

function cadi_kvec2vel, cadi, missing_vel=missing_vel

on_error, 0
float_missing = 1e9     ; 1000 MHz
lon_missing = long(float_missing)
nullresult = -1L

;%% Would have used the following, except we must know the type to return
;%% MISSING_VEL when no SOURCES_KVECTOR.
;%% fxrange: make_array(2, type=type(sources_kvector(0).x.f), value=-1), $
;%% hxrange: make_array(2, type=type(sources_kvector(0).x.h), value=-1), $

missing_vel = {cadi_velfit_stu, $
   time:      lon_missing, $
   nsources:  lon_missing, $
   fxrange:   replicate(255B,2), $
   hxrange:   replicate(255B,2), $
   azi:       float_missing, $
   hor:       float_missing, $
    horew:       float_missing, $
    horns:       float_missing, $
     horsun:     float_missing, $
     hordusk:    float_missing, $
   ver:       float_missing, $
   azi_err:   float_missing, $
   hor_err:   float_missing, $
   ver_err:   float_missing, $
   rms_error: float_missing}

if n_params() eq 0 then return, nullresult      ; Return MISSING_VEL only
if cadi.datatype ne 'KVEC' then $
   message, 'Argument does not have KVEC (k-vectors of sources) data'
nsources = cadi.ndata
if nsources eq 0 then $
   return, nullresult
header = cadi.header
sources_kvector = cadi.data
site = cadi_site(header.site, header.time)

;%% ; Find number of unique times (NTIMES) used by SOURCES_KVECTOR, and the
;%% ; indices of the first and last elements of SOURCES_KVECTOR for each unique
;%% ; time (vectors STARTX_UNIQTIME and ENDX_UNIQTIME).
;%% ;
;%% ; Assume times are in order, so d.x.t monotonic. UNIQ will then return the
;%% ; index into SOURCES_KVECTOR corresponding to the last element for each time.
;%%
;%% endx_uniqtime = array(uniq(array( sources_kvector.x.t )))
;%% ntimes = n_elements(endx_uniqtime)
;%% if ntimes gt 1 then $
;%%    startx_uniqtime = [0, endx_uniqtime(0:ntimes-2)+1] $
;%% else $
;%%    startx_uniqtime = [0]

time_blocks = cadi_tblocks(sources_kvector, ntimes=ntimes) ;%% Introduced 950208

if ntimes eq 0 then return, nullresult  ;%% NTIMES never zero here?

vel = replicate(missing_vel, ntimes)

for timex=0,ntimes-1 do begin
   ;%% print, timex, format='(i5,$)' ;%%
   ;%% startx = startx_uniqtime(timex)
   ;%% endx = endx_uniqtime(timex)
   startx = time_blocks(timex).startx
   endx = time_blocks(timex).endx
   nsources = endx - startx + 1
   vel(timex).time     = header.times(sources_kvector(startx).x.t)
   vel(timex).nsources = nsources
   min_fx = min(sources_kvector(startx:endx).x.f, max=max_fx)
   vel(timex).fxrange = [min_fx, max_fx]
   min_hx = min(sources_kvector(startx:endx).x.h, max=max_hx)
   vel(timex).hxrange = [min_hx, max_hx]

   ; Fit velocity if enough sources

   if nsources ge 3 then begin
      df = header.dops(sources_kvector(startx:endx).x.d)
      k = sources_kvector(startx:endx).k                ; Dims (3,NSOURCES)
      v = cadi_fit_vel(df, k, sing=sing, chisq=chisq, stddev=verr)
      if sing gt 0 then $
         message, /inform, $
         'Singular values in SVD for TIMEX = '+strim(timex)

      ; Calculate chisq without weighting

      chisq = 0.0
      for sx=0,nsources-1 do $
         chisq = chisq + (df(sx) - vect3_dot(v,k(*,sx))/!pi)^2

      ; Convert velocity to cylindrical coordinates, with errors

      v_hor = vect3_mag([v(0), v(1), 0])
      vel(timex).hor = v_hor
      vel(timex).azi = $
         posmod( (atan(v(0),v(1))+site.azimuth) *!radeg, 360.0)
      vel(timex).horew = v(0)
      vel(timex).horns = v(1)
      timeofday = double(vel(timex).time MOD 86400L)
      vel(timex).horsun = v_hor*cos(!dtor*(vel(timex).azi - 15.0*(timeofday - 21600.0)/3600))
      vel(timex).hordusk = -v_hor*sin(!dtor*(vel(timex).azi - 15.0*(timeofday - 21600.0)/3600))
      vel(timex).ver = v(2)
      vel(timex).hor_err = $
         (abs(v(0))*verr(0) + abs(v(1))*verr(1)) / v_hor
      vel(timex).azi_err = $
         (abs(v(1))*verr(0) + abs(v(0))*verr(1)) / (v_hor^2) * !radeg
      vel(timex).ver_err = verr(2)
      vel(timex).rms_error = sqrt(chisq/nsources)
      endif ; nsources ge 3

   endfor ; timex

print

return, vel
end
;+
; NAME:
;       CADI_KVEC_EDIT
; PURPOSE:
;       Reject CADI KVEC sources with angle of arrival outside limits.
; CALLING SEQUENCE:
;       Result = CADI_KVEC_EDIT( CADI, LIMITS_TAN=LIMITS_TAN )
; INPUTS:
;       CADI: Variable of CADI structure with KVEC data.
; KEYWORD PARAMETERS:
;       LIMITS_TAN = 4-element float vector giving limits of tangents of
;          the two angles which define an echo direction, as plotted by
;          CADI_SKYMAPS:
;             [ min tan_east, min tan_north, max tan_east, max tan_north ].
;          For no limit, set a 'min' element to less than or equal to -99, or
;          a 'max' element greater than or equal to +99.
;          Default is [-99,-99,99,99], i.e. return all sources.
;          Example 1: [-1,-1,1,1] would accept only those echoes less than 45
;          degrees east or west of the north-south meridian, and less than
;          45 degrees north or south of the east-zenith-west great circle.
;          Example 2: [-1,-99,0,99] would accept all sources west of the
;          meridian but within 45 degrees of it.
; OUTPUTS:
;       Result: Variable of CADI structure with KVEC data. Result.DATA is a
;          subset of CADI.DATA.
; PROCEDURE:
;       From the k-vector components kx, ky, kz, calculate
;       tan(theta_east) = kx / kz and tan(theta_north) = ky / kz and compare
;       with the limits.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 30 Jan 1995.
;-

function cadi_kvec_edit, cadi, limits_tan=limits_tan

on_error, 0

if not keywordset(limits_tan) then limits_tan = [-99.0, -99.0, 99.0, 99.0]
if cadi.datatype ne 'KVEC' then $
   message, 'Argument does not have KVEC (k-vector) data'
nsources = cadi.ndata
if nsources eq 0 then $
   goto, form_nullresult
header = cadi.header
d = cadi.data
site = cadi_site(header.site, header.time)

; Reject sources with k-vector outside at least one limit

nkvecsources = nsources
kz = d.k(2)
tan_e = d.k(0) / kz     ; kx / kz
tan_n = d.k(1) / kz     ; ky / kz
wgood = where( (tan_e ge limits_tan(0)) and $
               (tan_n ge limits_tan(1)) and $
               (tan_e le limits_tan(2)) and $
               (tan_n le limits_tan(3)), nsources)
;%% IDL requires different syntaxes for DIM(SOURCES) EQ/GT 1
;%% Until I think of a tidy workaround, don't process NSOURCES EQ 1
;%% 940909 Try to accept NSOURCES eq 1 %% if ngood le 1 then $  ;%% Added 940810
if nsources eq 0 then $
   goto, form_nullresult        ; All bins rejected
if nsources lt nkvecsources then $
   d = d(wgood)

form_result:

return, create_struct( $
   'HEADER_EXISTS', cadi.header_exists, $
   'NFREBINS', cadi.nfrebins, $
   'NDATA', nsources, $
   'DATATYPE', cadi.datatype, $
   'HEADER', cadi.header, $
   'FREBINS', cadi.frebins, $
   'DATA', d)

form_nullresult:

d = -1L
nsources = 0L
goto, form_result

end
