;+
; NAME:
;	CADI_TVFT
; PURPOSE:
;	Plot an image versus X (with gaps) and Y.
;	Plot an image Z with pixels located at the cartesian product of X and
;	Y, where the elements of X are mostly evenly spaced but may have a few
;	gaps. 
;	Pixels may have only two values: false or true (set).
; CALLING SEQUENCE:
;	CADI_TVFT, Z, X, Y
; INPUTS:
;	Z: Image of dimensions (NX, NY). Elements of Z are examined only to 
;	   determine whether they are true or false.
;	X: Vector of NX elements. Abscissa values for pixels in Z. The 
;	   sequence of values in X is typically regular, but may have
;	   arbitrarily many jumps between regular sections.
;	Y: Vector of NY elements. Ordinate values for pixels in Z. The 
;	   sequence of values in Y must be regular.
; KEYWORD PARAMETERS:
;	DX = Nominal step in X values. This is used to determine width of an 
;	   image of only one column.
;	DY = Nominal step in Y values. This is used to determine height of an 
;	   image of only one row.
;	FORE_COLOR = Color index to use for set (true) pixels.
;	BACK_COLOR = Color index to use for false pixels (background).
; PROCEDURE:
;	For each subimage between jumps in X, plot only the minimum
;	subrectangle which includes all set pixels.
;	   Pixel width (height) in data coordinates is the median step in X (Y)
;	if it has at least two elements, else DX (DY).
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 1 Nov 1993: Written.
;       GRANT,  5 Jan 1993: Various mods and bug fixes copied from PC to VAX
;          version.
;	GRANT, 22 Feb 1995: Added REFORM inside SUBRECT_WHERE call to handle 
;	   image of dimensions (1,1).
;	GRANT,  6 Mar 1995: Keywords DX or DY ignored if appropriate dimension
;	   of Z is not 1.
;-

pro cadi_tvft, z_in, x, y, dx=dx_nom_in, dy=dy_nom_in, $
   fore_color=fore_color, back_color=back_color

on_error, 2

nx = n_elements(x)
ny = n_elements(y)
size_z = size(z_in)
if howmany( size_z(0:2) ne [2, nx, ny]) gt 0 then $
   message, 'Dimensions of Z are not [ N_ELEMENTS(X), N_ELEMENTS(Y) ]'

if nx eq 1 then begin
   if not keyword_set(dx_nom_in) then $
      message, 'First dimension of Z is 1, so DX_NOM must be set' $
   else $
      dx_nom = dx_nom_in
   endif $
else begin
   dx = difference(x)
   dx_nom = median(dx)	; Nominal X step
   endelse

if ny eq 1 then begin
   if not keyword_set(dy_nom_in) then $
      message, 'Second dimension of Z is 1, so DY_NOM must be set' $
   else $
      dy_nom = dy_nom_in
   endif $
else begin
   dy = difference(y)
   dy_nom = median(dy)	; Nominal Y step
   endelse

; Identify upper side of each X jump J0, ..., Jj.
; Then plot blocks between
; jumps; X indices are [0 : J0-1], [J0 : J1-1], ..., [Jj:NX-1].

if nx eq 1 then $
   njumps = 0 $
else $
   wjumps = where(dx ne dx_nom, njumps)

if njumps eq 0 then $
   wjumps = [0, nx] $
else $
   wjumps = [0, wjumps+1, nx]

for imagex=0,njumps do begin
   st = wjumps(imagex)
   en = wjumps(imagex+1)-1

   ; Plot minimum subrectange which contains all set pixels.
   ; (Need REFORM to force argument of SUBRECT_WHERE to be 2D.)
   
   s = subrect_where(reform( z_in(st:en, *), en-st+1, size_z(2), /over ))
   if s(0) ne -1 then begin
      ; Image contains some set pixels. Plot it.
      z = 0		; Deallocate memory
      z = make_array( s(2)-s(0)+1, s(3)-s(1)+1, /byte, value=back_color)
      if (size(z))(0) ne 2 then $
         ; Ensure Z is 2D even if one or both dimensions are 1
         z = reform( temporary(z), s(2)-s(0)+1, s(3)-s(1)+1 )
      z(where( z_in(st+s(0):st+s(2), s(1):s(3)) )) = fore_color
      tvxy, z, x(st+s(0):st+s(2)), y(s(1):s(3)), dx=dx_nom, dy=dy_nom
      endif ; s(0) ne -1

   endfor ; imagex
return
end
