;+
; NAME:
;       CADI_HTV
; PURPOSE:
;       Plot CADI h't and/or velocity against time.
; CALLING SEQUENCE:
;       CADI_HTV, CADI_HT [,VEL=VEL]
; INPUTS:
;       CADI_HT = Variable of CADI structure which will supply the h't
;          information, and also ancillary information such as date, site and
;          sounding frequencies. CADI_HT.DATA need have only the X field.
;          Mandatory.
; KEYWORD PARAMETERS:
;       VEL = Vector of CADI velocity structures. Must be supplied if
;          PANELS specifies velocity parameters.
;       TIMERANGE = Two-element vector of CANOPUS times specifying start and
;          end time of plot.
;       PANELS = Scalar string with one character for each panel to be plotted.
;          Characters may be any combination of the following:
;             f   h't (specify frequency with FREQX_LIST)
;             A   Azimuth of horizontal component of velocity
;             H   Magnitude of horizontal component of velocity
;               X
;               Y
;             V   Vertical component of velocity
;             N   Number of sources in the velocity fit.
;             a   Error in azimuth of horizontal component of velocity
;             h   Error in magnitude of horizontal component of velocity
;             v   Error in vertical component of velocity
;       FREQX_LIST = Vector of indices into CADI_HT.HEADER.FREQS which
;          indicate which frequency to display in each h't panel specified by
;          PANELS. FREQX_LIST must have exactly one element for each 'f' in
;          PANELS.
;       ERROR: Integer scalar returned as nonzero if an error occurs.
;       INFORM: If present and nonzero, progress messages are printed.
;       HOR:       Forces y-range of 'H' panel to be [0, HOR].
;       VER:       Forces y-range of 'V' panel to be [-VER, VER].
;       EAZI:      Forces y-range of 'a' panel to be [0, EAZI].
;       EHOR:      Forces y-range of 'h' panel to be [0, EHOR].
;       EVER:      Forces y-range of 'v' panel to be [0, EVER].
;       NSOURCES:  Forces y-range of 'N' panel to be [0, NSOURCES].
;       POS:    4-element float vector of normalized coordinates specifying the
;               rectangle (x0,y0,x1,y1) on the display area in which to plot.
;               Default is [0.12, 0.1, 0.97, 0.95].
; RESTRICTIONS:
;       %% At present, all data might be plotted, rather than that within
;          TIMERANGE. Even past hour boundaries!!
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 19 Sep 1994: Adapted from CADI_MD_HTV.PRO.
;       GRANT, 2 Nov 1994: Added call to CADI_REDUCETIME to speed FHT h't plot.
;       GRANT, 9 Nov 1994: Added POS keyword.
;       GRANT, 15 Nov 1994: Put 'Err!c' into ytitle of each error trace.
;       GRANT, 11 Jan 1995: Changed CADI_HT from keyword to plain parameter.
;       GRANT, 8 Feb 1995: Added handling of null CADI_HT or VEL.
;       JMacD, 13 Apr 1995: Added pcl plotting
;       JMacD, 16 July 1996: added datadump
;       JMacD, 23 Sept 1997: added extended height
;		JMacD, 11 Dec 1997: datadump modified to dump xyz for Fortaleza
;		JMacD, 16 June 1999: datadump corrected to dump v(i).(4) for azimuth instead of y(i)
;-

pro cadi_htv, cadi_ht, vel=v_in, timerange=timerange, $
   panels=panels_str, freqx_list=htfreqx_list, max_height = maxheight, $
   error=error, inform=inform, $
   hor=hor, ver=ver, eazi=eazi, ehor=ehor, ever=ever, nsources=nsources, $
   pos=pos_regions, dumpdata=dumpdata
;stop
;%% VEL needs HEADER here only for:
;    - SITE for title
;    - Vel freq for title
;    - Default timerange

;%% Check h't, velocity and header info all come from the same site?

;%% Ensure only that subset of h't and vel within TIMERANGE are plotted.

on_error, 0 ;2

wild_char = '*'
max_value = 9.0e8       ; 900 MHz
min_azi = 0.0
if not keyword_set(panels_str) then panels_str = 'AHVNahv'
if not keyword_set(hor) then hor = 2000.0
if not keyword_set(ver) then ver = 500.0
if not keyword_set(eazi) then eazi = 50.0
if not keyword_set(ehor) then ehor = 90.0
if not keyword_set(ever) then ever = 50.0
if not keyword_set(nsources) then nsources = 100 ;200
if not keyword_set(pos_regions) then pos_regions = [0.12, 0.1, 0.97, 0.95]

if n_params() lt 1 then $
   message, 'Must supply argument CADI_HT'
if not cadi_ht.header_exists then $
   message, 'CADI_HT.HEADER does not exist'

;%% ht = cadi_ht.data
h = cadi_ht.header

; Find TIMERANGE, REFTIME for time axis, PLOT_TIMERANGE

if not keyword_set(timerange) then $
   timerange = canmidnight(h.time) + [0,24*3600L]
if timerange(1) - timerange(0) le 6*3600 then $
   reftime = canmidnight(timerange(0)) $
else $
   reftime = canjan0(timerange(0))
plot_timerange = timerange - reftime
nodaylabels = (timerange(1) le (canmidnight(timerange(0)) + (24*3600L)))

; In PANEL_STU, TAGX is index to frequency for h't, otherwise index of tag in V
; structure.

dummy = {panel_stu, $
   ytitle:'', code:' ', type:'', tagx:0L, tagname:'', yrellen:0.0, yrange:[0.0,0.0]}

panel_options = [ $
   ;{panel_stu, "Height!c(km)",         'f', 'HT',  0,  '',        4, [0,510]}, $
   {panel_stu, "Group Range!c(km)",         'f', 'HT',  0,  '',        4, [0,maxheight]}, $
   {panel_stu, 'V_azi!c(deg)',         'A', 'DEG', 0, 'AZI',      4, [min_azi,min_azi+360]}, $
   {panel_stu, 'V_hor!c(m/s)',         'H', '',    0, 'HOR',      4, [0,hor]}, $
   {panel_stu, 'V_ver!c(m/s)',         'V', '',    0, 'VER',      4, [-ver,ver]}, $
   {panel_stu, 'V_horEW!c(m/s)',       'X', '',    0, 'HOREW',    4, [-hor,hor]}, $
   {panel_stu, 'V_horNS!c(m/s)',       'Y', '',    0, 'HORNS',    4, [-hor,hor]}, $
   {panel_stu, 'Number!cof!csources',  'N', 'NOS',    0, 'NSOURCES', 1, [0,nsources]}, $
   {panel_stu, 'Err!cAzi',             'a', '',    0, 'AZI_ERR',  1, [0,eazi]}, $
   {panel_stu, 'Err!cHor',             'h', '',    0, 'HOR_ERR',  1, [0,ehor]}, $
   {panel_stu, 'Err!cVer',             'v', '',    0, 'VER_ERR',  1, [0,ever]} $
      ]

npanels = strlen(panels_str)
panels = replicate({panel_stu}, npanels)
for panelx=0,npanels-1 do begin
   panel_code = strmid(panels_str, panelx, 1)
   panel_optionx = (where(panel_code eq panel_options.code)) (0)
   if panel_optionx lt 0 then begin
      message, /inform, 'Unrecognized code in PANELS keyword'
      goto, error_return
      endif
   panels(panelx) = panel_options(panel_optionx)
   endfor

htpanelx_list = where(panels.type eq 'HT', npanels_ht)
vpanelx_list = where(panels.type ne 'HT', npanels_v)

if npanels_ht gt 0 then begin
   ;%% if not keyword_set(cadi_ht) then begin
   ;%%    message, /inform, $
   ;%%       'Keyword PANELS_STR='+panels_str+ $
   ;%%       ' requires keyword CADI_HT must be given'
   ;%%    goto, error_return
   ;%%    endif
   if not keyword_set(htfreqx_list) then $
      htfreqx_list = reverse(lindgen(npanels_ht))

   ; Check h't frequency list is valid.

   if n_elements(htfreqx_list) ne npanels_ht then begin
      message, /inform, $
         'Number of HTFREQX_LIST elements must equal number of "f" panels'
      goto, error_return
      endif
   if (howmany(htfreqx_list gt h.nfreqs-1) gt 0) or $
      (howmany(htfreqx_list lt 0) gt 0) then begin
      message, /inform, $
        'HTFREQX_LIST elements are out of range; max allowed is '+strim(h.nfreqs-1)
      goto, error_return
      endif

   ; Store frequency list in the TAG field of the h't PANELS.

   if npanels_ht gt 1 then $
      panels(where(panels.type eq 'HT')).tagx = htfreqx_list $  ; LHS a vector
   else $
      panels(where(panels.type eq 'HT')).tagx = htfreqx_list(0) ; LHS a scalar
   endif

if npanels_v gt 0 then begin
   if not keyword_set(v_in) then begin
      message, /inform, $
         'Keyword PANELS_STR='+panels_str+' requires keyword VEL must be given'
      goto, error_return
      endif
   v = v_in
   if (size(v_in)) (0) eq 0 then begin

      ; No elements in velocity argument, so create two missing velocities to
      ; avoid problems.

     dummy = cadi_kvec2vel(missing=v_missing)
     v = replicate(v_missing, 2)
     v_missing.time = 0L  ; timerange
      endif
   v_tagnames = tag_names(v)
   for panelxx=0,npanels_v-1 do begin
      panelx = vpanelx_list(panelxx)
      ; Set tag numbers from the velocity argument itself, in case the
      ; velocity structure changes in future.
      panels(panelx).tagx = (where(v_tagnames eq panels(panelx).tagname)) (0)
      endfor
   endif ; npanels_v gt 0

; Temporary list of Y norm coords of panel horizontal edges.
; First panel (panelx=0) is at the top.

pos = pos_regions(3) - (pos_regions(3) - pos_regions(1)) * $
   (cumulative_sum([0, panels.yrellen]) / total(panels.yrellen))
; Each row will be the position of the panel corners in norm coordinates.
pos_list = transpose( [ $
         [replicate(pos_regions(0), npanels)], $
         [pos(1:npanels)], $
         [replicate(pos_regions(2), npanels)], $
         [pos(0:npanels-1)] $
            ] )

psav = !p
xsav = !x
ysav = !y
pticklen = !p.ticklen

if npanels_v gt 0 then begin

   ; Nonmissing drift points

   w_nonmiss = where(v.azi lt 400.0, nonmiss_count) &&

   ; Put drift freq into string

   if nonmiss_count gt 0 then begin
      min_fx = min(v(w_nonmiss).fxrange(0))
      max_fx = max(v(w_nonmiss).fxrange(1))
      if min_fx eq max_fx then $
         vfreq_str = strim(h.freqs(min_fx) / 1e6, format='(f6.3)') + ' MHz' $
      else $
         vfreq_str = 'Multi-freq'
      endif $
   else $
      vfreq_str = '0.000' + ' MHz'

   endif ; npanels_v gt 0

; Start plot

erase

; Set up to plot h't images.

if npanels_ht gt 0 then begin

   ; Set up device for images

   if !d.name eq 'PS' then begin
      device, bits_per_pixel=1 ;%% Better without DEVICE call in this procedure?
      back_color = 255b ; White
      fore_color = 0b   ; Black
   endif else if !d.name eq 'PCL' then begin
      back_color = 255b ; White
      fore_color = 0b   ; Black
   endif else begin
      back_color = 0b   ; White
      fore_color = 255b ; Black
      endelse

   ; Determine default pixel height DH.

   dh = median(difference( h.heights ))         ; Typical height step.

   endif ; npanels_ht gt 0

; Plot each panel

for panelx=0,npanels-1 do begin
   panel = panels(panelx)
   pos = pos_list(*,panelx)

   if panelx eq 0 then begin
      title_ht = ''
      title_and = ''
      title_v = ''
      if npanels_ht gt 0 then $
         title_ht = " h't"
      if npanels_ht gt 0 and npanels_v gt 0 then $
         title_and = ' and'
      if npanels_v gt 0 then $
         title_v = ' drift (' + vfreq_str +')'
      title = 'CADI' + title_ht + title_and + title_v + '    ' + $
         h.site + '    ' + strcantime(timerange(0),/date)
      endif $
   else $
      title = ''

   case panel.type of
      'HT': begin

         ; Define axes of subplot without drawing
;stop
         plot, /noerase, /nodata, [0], [0], pos=pos, $
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ystyle=5

         ;oplot, [71700,71700],[0,510],linestyle=2
         ;oplot, [73500,73500],[0,510],linestyle=2
         ;oplot, [75300,75300],[0,510],linestyle=2
   ; Find pixels for this frequency ;%% and time range

         ;%% This SUB line takes ~80% of time before image plotting starts
         ;%% ht_onef = cadi_sub(ht, frange=[panel.tagx,panel.tagx], wcount)
         cadi_ht_onef = $
            cadi_sub_struct(cadi_ht, frange=[panel.tagx,panel.tagx])
         ;%% if wcount gt 0 then begin
stop
         if cadi_ht_onef.ndata gt 0 then begin

            ; Some pixels are set for this frequency and hour.

            ; If FHT, keep only the times with data

            if h.file_ext eq 'FHT' then $
               cadi_ht_onef = cadi_reducetime(cadi_ht_onef)

            ;%% Extract HEADER and DATA fields for this code to plot the image

            ;%% h_onef = cadi_ht_onef.header
            ;%% ht_onef = cadi_ht_onef.data

            ; Determine default pixel width DT.

            if h.file_ext eq 'FHT' then $
               dt = h.nfreqs * h.dtime $
            else $
               dt = h.dtime

            ; Find smallest rectangle on t-h plane which includes all set pixels.

            ; %% This was patched together from existing bits of code. It would
            ; probably be more efficient of space to construct the image only for
            ; each inter-jump interval at a time, instead of the whole Z at once
            ; as done here. The MIN_XT etc. stuff is not strictly necessary here
            ; since something similar is done inside CADI_TVFT, but I leave it
            ; here to reduce the size of the complete Z.

            min_xt = min(cadi_ht_onef.data.x.t, max=max_xt)
            min_xh = min(cadi_ht_onef.data.x.h, max=max_xh)
            tdim = max_xt - min_xt + 1
            hdim = max_xh - min_xh + 1

;%% From SUB line to here takes ~10% of time before image plotting starts

            ; Create an image of this smallest rectangle and plot it

            z = 0             ; Deallocate memory
            z = make_array( tdim, hdim, /byte)
            if (size(z))(0) ne 2 then $
               ; Ensure Z is 2D even if one or both dimensions are 1
               z = reform( temporary(z), tdim, hdim )
            z( cadi_ht_onef.data.x.t-min_xt, cadi_ht_onef.data.x.h-min_xh ) = 1B
            cadi_tvft, z, $
               cadi_ht_onef.header.times(min_xt:max_xt)-reftime, $
               cadi_ht_onef.header.heights(min_xh:max_xh), $
               dx=dt, dy=dh, fore_color=fore_color, back_color=back_color
            endif ; wcount gt 0

         ; Plot axes only for h't. Image has been plotted already.

         htfreq_str = strim(h.freqs(panel.tagx) / 1e6, format='(f6.3)') + ' MHz'
         plot, [0], [0], /nodata, $
            /noerase, pos=pos, title=title, $
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ytitle=htfreq_str+'!c'+panel.ytitle, ystyle=1

         end
      'DEG': begin
         ; Azimuth

         ; Move angle (azimuth) into range [min_azi, min_azi+360.0]

         y0 = v.(panel.tagx)
         if nonmiss_count gt 0 then $
            y0(w_nonmiss) = posmod(y0(w_nonmiss)-min_azi,360.0)+min_azi

         ; Prepare to gracefully plot 360 degree jumps

         wrap_preplot, v.time, y0, min_azi, min_azi+360.0, $
            t, y, missing=1.1*max_value

         plot, [t-reftime], [y], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1, $
            yticks=4, yminor=9
         end
    'NOS': begin
         plot, [v.time-reftime], [v.(panel.tagx)], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1, $
            yticks=2
         end
      else: begin
        if keyword_set(dumpdata) then begin
        ;stop
          if ((panel.tagx eq 5)OR(panel.tagx eq 6)) then begin
          print, "making data file from #" , panel.tagx
           openw, 1, 'a:output.dat'
             n = n_elements(v)
             printf, 1, n
             for i = 0,n-1 do begin
              if (panel.tagx eq 5) then printf, 1, v(i).time-reftime, v(i).(4), v(i).(panel.tagx),v(i).ver , v(i).azi_err, v(i).hor_err $
              else printf, 1, v(i).time-reftime, v(i).horew, v(i).horns, v(i).ver
             ;stop
             endfor
           close, 1
        endif
        endif
         plot, [v.time-reftime], [v.(panel.tagx)], $
            /noerase, max_value=max_value, pos=pos, psym=-3, title=title, $
            xrange=plot_timerange, xstyle=5, $
            yrange=panel.yrange, ytitle=panel.ytitle, ystyle=1
         oplot, plot_timerange, [0,0]
         end
      endcase

   ; Plot timeaxes

   top = (panelx eq 0)
   nolabels = (panelx ne npanels-1)
   if nolabels then $
      xtitle = ' ' $
   else $
      xtitle = ''       ; Use default
   !p.ticklen = pticklen / (pos(3) - pos(1))
   !x.charsize = 1.25   ; Compensate CHARSIZE=0.8 in TIMESCALE
   timescale, plot_timerange(0), plot_timerange(1), title=xtitle, $
      top=top, nolabels=nolabels, tic_factor=-1, nodaylabels=nodaylabels
   !p.ticklen = pticklen

   endfor ; panelx

!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
return

end
