;+
; NAME:
;       CADI_IGSUMM
; PURPOSE:
;       Generate summary plot of one day's ionograms from CADI data files.
;       Designed for MD and FFI data formats.
; CALLING SEQUENCE:
;       CADI_IGSUMM, DIR=DIR, DATESTR=DATESTR, EXT=EXT
; INPUTS:
;       DIR = Scalar string containing directory in which data is located.
;          Syntax ready to prepend onto filename e.g. 'D:\CADIDATA\'.
;          Default is ''.
;       DATESTR = string holding date of data as 6 ASCII digits YYMMDD.
;          Default is to take date from header of first .ext file in directory.
;       EXT = scalar string containing extension of data files, without period.
;          Default is 'MD0'.
; KEYWORD PARAMETERS:
;       INFORM: If present and nonzero, progress messages are printed.
;       ERROR = integer scalar. Returned as true if an error occurred, else
;          false.
; SIDE EFFECTS:
;       Generates a plot file.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 3 Feb 1995.
;       GRANT, 6 Mar 1995: Trimmed white space from around ionograms by SUBRECT.
;       MacDougall, 22 May 1995: Commmented out checking for logarthmic frequency scale
;		MacDougall, 21 Feb 1997: added mods so can do all ionograms in one hour
;		30 Jan 2000 by JWM Y2K fix to year
;		By jwm 9jun01 to plot 1 ionogram per hour using keyword 'hourly'
;  		jwm 28 June 2001 mod to plot electron densities for el gradient calcs
;			keywords added: eldensityrange, heightrange (both 2 element arrays)
;-

pro cadi_igsumm, dir=dir, datestr=datestr_in, ext=file_ext, $
   inform=inform, error=error, part_day = part_day, thour = thour, heightrange = heightrange, htlimit=heightlimit,$
    hourly = hourly, eldensityrange = eldensityrange, noofcolumns = noofcolumns, noofrows = noofrows, $
       cantimein = cantimein

on_error, 2

psav = !p
xsav = !x
ysav = !y
;stop
; Set up plot now to be ready for any 'Aborted' message.

erase
time

if not keyword_set(file_ext) then file_ext = 'MD0'
if not keyword_set(dir) then dir = ''

; Get header information (CADI_READ_DIR get it from first file)

h = cadi_read_dir(dir=dir, ext=file_ext, /onlyheader, error=error )
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif

; Input date from keyword or from first file

if not keyword_set(cantimein) then begin
if not keyword_set(datestr_in) then $
   candate = canmidnight(h.time) $   ; Date from time in first file
else begin
   datestr = datestr_in ; Unnecessary?
   reads,datestr,year,month,day,format='(3i2)'       ; Date from keyword
   if year lt 54 then year = year + 100        ;y2k adjustment
   ;print, 'doing timecalc'
   candate=cantime(1900+year,month,day,0,0,0)
   ;print, 'done'
   endelse
; Get DAYNUM for plot title

civil_time, year, month, day, daynum, hour, minute, second, can_time = candate

endif else begin
     cantimeset = cantimein
     sthr = thour
     noofhours = (cantimeset(1) - cantimeset(0))/3600L
;print, 'noofhours = ',noofhours
     endhr = sthr + noofhours - 1
endelse

rx = 0L         ; Index of receiver to be plotted
mhzrange = [1.0,20.0]
;if not keyword_set(part_day) then begin
;if not keyword_set(hourly) then begin
; nrows = 4L
; ncols = 12L
; max_npanels = 2
; sthr = 0
; endhr = 23
; tdiff = 15
;endif else begin ;hourly
; nrows = 1L
; ncols = 6L
; max_npanels = 4
; sthr = 0
; endhr = 23
; tdiff = 60
; endelse
;endif else begin ;part_day
;if part_day eq 1 then begin
; if h.dtime eq 30 then begin
; nrows = 6L ;4L
; ncols = 10L ;5L
; max_npanels = 1
; sthr = thour
; endhr = thour; + 5
; tdiff = 1 ;0.5 ;5 ;1
; endif
;if h.dtime eq 60 then begin
; nrows = 6L ;4L
; ncols = 10L ;5L
; max_npanels = 1
; sthr = thour
; endhr = thour; + 5
; tdiff = 1.0 ;5 ;1
; endif
;if h.dtime eq 120 then begin
; nrows = 6L ;4L
; ncols = 5L ;5L
; max_npanels = 1
; sthr = thour
; endhr = thour;
; tdiff = 2.0
; endif
;endif ;part_day eq 1
;if part_day eq 4 then begin
;nrows = 3L
;ncols = 5L
;max_npanels = 1
;sthr = thour
;endhr = thour
;tdiff = 1.0
;endif
;endelse
nrows = noofrows
;print, 'no of rows = ',nrows
ncols = noofcolumns
max_npanels = 1 		;may need to adjust this
times_per_panel = nrows*ncols
tdiff = noofhours*60/times_per_panel
;print, 'tdiff = ',tdiff
!p.noerase = 1
!p.ticklen = 0.04
charsize = 0.8

; can next use pos_regions (array variable) to divide up the plot into a number of regions
;for instance if not set hourly and set part_day get just 1 region
;if not set hourly and not set part_day get upper and lower regions
if not keyword_set(hourly) then begin
  pos_regions = fltarr(4,2)
  if not keyword_set(part_day) then begin
    pos_regions(0,0) = [0.05, 0.55, 1.0, 0.90]
    pos_regions(0,1) = pos_regions(*,0) - 0.48*[0, 1, 0, 1]
  endif else pos_regions(0,0) = [0.05, 0.1, 0.98, 0.90]
endif else begin ;hourly
  pos_regions = fltarr(4,4)
  pos_regions(0,0) = [0.05,0.75,1.0,0.93]
  pos_regions(0,1) = pos_regions(*,0) - 0.24*[0,1,0,1]
  pos_regions(0,2) = pos_regions(*,1) - 0.24*[0,1,0,1]
  pos_regions(0,3) = pos_regions(*,2) - 0.24*[0,1,0,1]
endelse



pos_in_region = [0, 0, 1, 1]
def_ticknames = replicate('',20)
blank_ticknames = replicate(' ',20)

; Start plot

;%% repeat begin ; until hardcopy()

if !d.name eq 'PS' then begin
   device, bits_per_pixel=1
   back_color = 255b    ; White
   fore_color = 0b      ; Black
 endif else if !d.name eq 'PCL' then begin
      back_color = 255b ; White
      fore_color = 0b   ; Black
  endif else begin
   back_color = 0b      ; black
   !p.background = 0b
   fore_color = 255b    ; white
   !p.color = 255b
 endelse
;erase
prev_timex = -1
prev_panelx = -1

; Plot title above position for top panel
;the following procedure makes the panels for the plots
region_multi, /init, [ncols,nrows], pos=pos_regions(*,0), $
   title='CADI ' + h.site + " Ionograms    " + strcantime(/date,cantimeset(0)) ;+ $
   ;'    Day ' + string(daynum, format='(i3)')
region_multi, /reset

; Loop through hours
if dir eq "md4" then htlimit = 1020 else htlimit = 510
if keyword_set(heightrange) then htlimit = heightrange(1)
;stop

for hourx=sthr,endhr do begin
;print,'hour =',hourx
;if keyword_set(part_day) then begin
;if part_day eq 4 then timerange_hour = candate + 3600L*hourx + [0,899L] else $
 ;  timerange_hour = candate + 3600L*hourx + [0,3599L]
;endif else timerange_hour = candate + 3600L*hourx + [0,3599L]
;print, 'yes'
;stop
timerange_hour = cantimeset(0) + (hourx-sthr)*3600L + [0,3599L]

   cadi = cadi_read_dir(dir=dir, timerange=timerange_hour, $
      ext=file_ext, /tfh, error=read_error, inf=keyword_set(inform), htlimitmax=htlimit )
   if read_error then begin
      message, /inform, $
         'Error reading data in time range '+strcanrange(timerange_hour)
      goto, end_hourx_loop
      endif
;print, 'here'
   if not cadi.header_exists then $
      goto, end_hourx_loop      ; No soundings for this hour
   h = cadi.header
   d = cadi.data
;stop
   dh = median(difference( h.heights )) ; Typical height step; pixel height
;print, 'here 0'
   ; Check frequency scale is close to logarithmic, and plot as though it is.
   ; Typically, most points will lie right on the log curve.

   logf = alog10(h.freqs)
   dlogf = logf(1:h.nfreqs-1)-logf(0:h.nfreqs-2)
   m = median(dlogf)            ; Typical logf step, i.e. pixel separation
   fx = indgen(h.nfreqs)
   c = median(logf - m*fx)      ; Typical intercept of logf(i) vs i
   logfnom = m*fx+c             ; Line of typical logf(i) vs i
   errlogf = (logf-logfnom)/m   ; Compare errors with pixel width M
;   if max(abs(errlogf)) gt 0.5 then begin ; Tolerance is half a pixel width
;      error_message = 'Frequency scale is too far off logarithmic for hour '+ $
;         strcanrange(timerange_hour)
;      goto, error_return
;      endif
   if not keyword_set(eldensityrange) then fnom = 10.0^(logfnom-6.0)  else begin $  ; Nominal frequencies in MHz
			fnom = h.freqs*h.freqs/80.5
			nnom = n_elements(fnom)
			dfnom = fnom(1:nnom-1)-fnom(0:nnom-2)
			m = median(dfnom)
            endelse
   for htimex=0,h.ntimes-1 do begin
      ;print, 'here 1'
      civil_time, year, month, day, daynum, hour, minute, second, h.times(htimex)
   if not keyword_set(part_day) then $
         timex = round( (60*hour+minute) / tdiff ) $ ; Nearest 15 minute time slot
   ;else timex = round(minute/ tdiff)
   else timex = round( (60*(hour-sthr)+minute) / tdiff )
      panelx = timex/times_per_panel            ; Panel holding that time slot

      ; Don't assign more than one ionogram to a time slot, or past last slot
      ;stop
      if (timex gt prev_timex) and (timex lt max_npanels*times_per_panel) then $
         begin
         ;stop
         ; Set up a new panel of ionograms if needed
         if panelx gt prev_panelx then begin
            if prev_panelx gt -1 then region_multi, /reset
            region_multi, /init, [ncols,nrows], pos=pos_regions(*,panelx)
            panel_stimex = timex
            endif

         ; Set-up axes location

         subplotx = timex mod times_per_panel
         subplot_col = subplotx/nrows           ; Left column is 0
         subplot_row = subplotx mod nrows       ; Top row is 0
         region_multi, [subplot_col, subplot_row]
         region_position, pos_in_region
         ;print,'axis plot'

         hrange = ends(h.heights)
         if keyword_set(heightrange) then hrange = heightrange
         if hrange(1) gt htlimit then hrange(1) = htlimit
         ;stop
         if not keyword_set(eldensityrange) then plot_oi, /nodata, /noerase, [0], [0], $
            xrange=mhzrange, xstyle=5, yrange=hrange, ystyle=5 else $
		plot, /nodata, /noerase, [0], [0], xrange = eldensityrange, xstyle = 5, $
		       yrange = hrange, ystyle = 5
         ; Plot ionogram, if any echoes (uniform steps in log(frequency)).

         if cadi.ndata gt 0 then begin          ; Any echoes in this hour?
            w = where(d.x.t eq htimex, wcount)  ; Any echoes for this time?
            if wcount gt 0 then begin
               d_ionogram = d(w)
			;if keyword_set(eldensityrange) then

              ; Find smallest rectangle containing all set pixels
               min_xf = min(d_ionogram.x.f, max=max_xf)
               min_xh = min(d_ionogram.x.h, max=max_xh)
               fdim = max_xf - min_xf + 1
               hdim = max_xh - min_xh + 1

               ; Create an image of this smallest rectangle and plot it

               z = 0             ; Deallocate memory
               z = make_array( fdim, hdim, /byte, value=back_color)
              ;z = make_array( fdim, hdim, /int, value=back_color)
               if (size(z))(0) ne 2 then $
                  ; Ensure Z is 2D even if one or both dimensions are 1
                  z = reform( temporary(z), fdim, hdim )
               z( d_ionogram.x.f-min_xf, d_ionogram.x.h-min_xh ) = fore_color
               ;stop
               tvxy, z, fnom(min_xf:max_xf), h.heights(min_xh:max_xh), $
                  dx=m, dy=dh
               ;%% cadi_tvft, z, $
               ;%%    cadi.header.freqs(min_xf:max_xf) - candate, $
               ;%%    cadi.header.heights(min_xh:max_xh), $
               ;%%    dx=m, dy=dh, fore_color=fore_color, back_color=back_color

               endif ; wcount gt 0
            endif ; cadi.ndata gt 0

         ; Plot axes

         if subplot_row eq nrows-1 then begin
           if not keyword_set(eldensityrange) then begin
            xtitle = 'MHz'
            xtickname = def_ticknames
            endif else xtitle = 'el/m^3'
         endif else begin
            xtitle = ''
            xtickname = blank_ticknames
            endelse
         if (timex lt panel_stimex+nrows) then begin
            ytitle = 'km'
            ytickname = def_ticknames
         endif else begin
            ytitle = ' '
            ytickname = blank_ticknames
            endelse
         if not keyword_set(eldensityrange) then plot_oi, /nodata, /noerase, [0], [0], $
            xrange=10.0^!x.crange, xstyle=1, xtitle=xtitle, xtickname=xtickname, $
            yrange=!y.crange, ystyle=1, ytitle=ytitle, ytickname=ytickname, $
            yminor=2 else $
            plot, /nodata, /noerase, [0], [0], $
             xrange = eldensityrange, xstyle = 1, xtitle = xtitle, $
             yrange = !y.crange, ystyle = 1, ytitle = ytitle, ytickname = ytickname, $
             yminor = 2, xticks = 2, xcharsize = 0.5

         ; Write time on ionogram

         hhmm_str = string(hour, minute, format='(2i2.2)')
         if not keyword_set(eldensityrange) then xyouts, 1.3, 50, '!c'+hhmm_str, $
            size=charsize else xyouts, (eldensityrange(0)+eldensityrange(1))/2.0, $
              heightrange(1) - 10, '!c' +hhmm_str, size = charsize/1.5

         prev_timex = timex
         prev_panelx = panelx
         endif ; timex gt prev_timex

      endfor ; timex
   end_hourx_loop:
   endfor ; hourx

region_multi, /reset
;%% time
;%% endrep until hardcopy()

!p.noerase = 0
!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
if not keyword_set(error_message) then error_message = ''
close, /all
region_multi, /clear
message, /inform, 'Aborted. '+error_message
xyouts, 0.3, 0.04, /norm, "Aborted ionogram summary plot for " + $
   datestr_in + ', ' + dir + '*.' + file_ext + $
   '!c' + error_message
return

end
