;+
; NAME:
;       CADI_IGWSUMM
; PURPOSE:
;       Generate summary plots of ionograms from CADI data files.
;       Designed for MD and FFI data formats.
; CALLING SEQUENCE:
;       CADI_IGWSUMM, DIR=DIR, DATESTR=DATESTR, EXT=EXT
; INPUTS:
;       DIR = Scalar string containing directory in which data is located.
;          Syntax ready to prepend onto filename e.g. 'D:\CADIDATA\'.
;          Default is ''.
;       DATESTR = string holding date of data as 6 ASCII digits YYMMDD.
;          Default is to take date from header of first .ext file in directory.
;       EXT = scalar string containing extension of data files, without period.
;          Default is 'MD0'.
;		THOUR = gives starting hour for the ionograms (if cnatimein it is derivedfrom the cantime)
;		HEIGHTRANGE = 2 element array giving min height and max height
;		ELDENSITYRANGE = 2 element array giving range if x scale is electron density (not implemented in cewidgets)
;		NOOFPANELS, NOOFCOLUMNS< NOOFROWS: gives geometry of the plotting
;		CANTIMEIN = 2 element array of type Cantime giving beginning and end of plottimes
;		COLOUR = if set, plots in color
; KEYWORD PARAMETERS:
;       INFORM: If present and nonzero, progress messages are printed.
;       ERROR = integer scalar. Returned as true if an error occurred, else
;          false.
; SIDE EFFECTS:
;       Generates a plot file.
; MODIFICATION HISTORY:
;       JWM 4 April 2004 program is based on Ian Grant's cadi_igsumm.
;-

pro cadi_igwsumm, dir=dir, datestr=datestr_in, ext=file_ext, $
   inform=inform, error=error, thour = thour, heightrange = heightrange,$
    eldensityrange = eldensityrange, noofpanels = noofpanels, noofcolumns = noofcolumns, $
    noofrows = noofrows, cantimein = cantimein, colour = colour

on_error, 0 ;2

psav = !p
xsav = !x
ysav = !y
;stop
; Set up plot now to be ready for any 'Aborted' message.

erase
;time

if not keyword_set(file_ext) then print, 'no file extension spaecified'
if not keyword_set(dir) then dir = ''

; Get header information (CADI_READ_DIR get it from first file)

h = cadi_read_dir(dir=dir, ext=file_ext, /onlyheader, error=error )
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif

; Input date from keyword or from first file

if not keyword_set(cantimein) then begin
  if not keyword_set(datestr_in) then $
   candate = canmidnight(h.time) $   ; Date from time in first file
  else begin
   datestr = datestr_in ; Unnecessary?
   reads,datestr,year,month,day,format='(3i2)'       ; Date from keyword
   if year lt 54 then year = year + 100        ;y2k adjustment
   ;print, 'doing timecalc'
   candate=cantime(1900+year,month,day,0,0,0)
   ;print, 'done'
   endelse
; Get DAYNUM for plot title

  civil_time, year, month, day, daynum, hour, minute, second, can_time = candate

endif else begin   ;yes there is cantimein
     cantimeset = cantimein
     civil_time, year, month, day, daynum, starthour, minute, second, cantimein(0)
     sthr = starthour
     noofhours = (cantimeset(1) - cantimeset(0))/3600L
;print, 'noofhours = ',noofhours
     ;endhr = sthr + noofhours - 1
endelse

rx = 0L         ; Index of receiver to be plotted
mhzrange = [1.0,20.0]
nrows = noofrows
;print, 'no of rows = ',nrows
ncols = noofcolumns
max_npanels = 1 		;may need to adjust this
times_per_panel = nrows*ncols
tdiff = noofhours*60/(times_per_panel*noofpanels)
;print, 'tdiff = ',tdiff
!p.noerase = 1
!p.ticklen = 0.04
charsize = 0.8
;stop
if (noofpanels eq 1) then begin 			;just 1 plotting region with this size
  if keyword_set(colour) then begin
    pos_regions = fltarr(4,2)
    pos_regions(0,0) = [0.05, 0.08, 0.93, 0.95]
    pos_regions(0,1) = [0.93, 0.08,0.98,0.95]
  endif else begin
    pos_regions = fltarr(4,1)
    pos_regions(0,0) = [0.05, 0.1, 0.98, 0.90]
  endelse
endif else begin		;2 plotting panels
  if keyword_set(colour) then begin
    pos_regions = fltarr(4,3)
    pos_regions(0,0) = [0.05, 0.55, 0.93, 0.90]
    pos_regions(0,1) = pos_regions(*,0) - 0.48*[0, 1, 0, 1]
    pos_regions(0,2) = [0.93,0.08,0.98,0.95]
  endif else begin
    pos_regions = fltarr(4,2)					;upper and lower plotting regions
    pos_regions(0,0) = [0.05, 0.55, 1.0, 0.90]
    pos_regions(0,1) = pos_regions(*,0) - 0.48*[0, 1, 0, 1]
  endelse
endelse

pos_in_region = [0, 0, 1, 1]
def_ticknames = replicate('',20)
blank_ticknames = replicate(' ',20)

; Start plot

;if keyword_set(colour) then begin
   !p.background = 255b		;white
   back_color = 255b
   if keyword_set(colour) then !p.color = 0 else !p.color = 255 				;black
;   stop
;   print, 'p.color = ', !p.color
   fore_color = 0b
;endif ;else begin
 ;   back_color = 0b      ; White
  ; fore_color = 255b    ; Black
;endelse
erase

prev_timex = -1
prev_panelx = -1

; Plot title above position for top panel
;the following procedure makes the panels for the plots
region_multi, /init, [ncols,nrows], pos=pos_regions(*,0),$
   title='CADI ' + h.site + " Ionograms    " + strcantime(/date,cantimeset(0)) ;+ $
   ;'    Day ' + string(daynum, format='(i3)')
region_multi, /reset

; Loop through hours
if dir eq "md4" then htlimit = 1020 else htlimit = 510
if keyword_set(heightrange) then htlimit = heightrange(1)
;stop

;;for hourx=sthr,endhr do begin
 ;; timerange_hour = cantimeset(0) + (hourx-sthr)*3600L + [0,3599L]
if keyword_set(colour) then tfhset = 0 else tfhset = 1
;stop
cadi = cadi_read_dir(dir=dir, timerange = cantimeset,$ ;timerange=timerange_hour, $
    ext=file_ext, tfh=tfhset, error=read_error, inf=keyword_set(inform), htlimitmax=htlimit )
;stop
if read_error then begin
    message, /inform, $
      'Error reading data in time range '+strcanrange(timerange_hour)
    goto, end_hourx_loop
endif

if not cadi.header_exists then goto, end_hourx_loop      ; No soundings for this hour

   h = cadi.header
   d = cadi.data

   ;stop
   dh = median(difference( h.heights )) ; Typical height step; pixel height

   ; Check frequency scale is close to logarithmic, and plot as though it is.
   ; Typically, most points will lie right on the log curve.

   logf = alog10(h.freqs)
   dlogf = logf(1:h.nfreqs-1)-logf(0:h.nfreqs-2)
   m = median(dlogf)            ; Typical logf step, i.e. pixel separation
   fx = indgen(h.nfreqs)
   c = median(logf - m*fx)      ; Typical intercept of logf(i) vs i
   logfnom = m*fx+c             ; Line of typical logf(i) vs i
   errlogf = (logf-logfnom)/m   ; Compare errors with pixel width M
;   if max(abs(errlogf)) gt 0.5 then begin ; Tolerance is half a pixel width
;      error_message = 'Frequency scale is too far off logarithmic for hour '+ $
;         strcanrange(timerange_hour)
;      goto, error_return
;      endif
prevf = 0.0
prevh = 0
prevpowdb = 0.0
   if not keyword_set(eldensityrange) then fnom = 10.0^(logfnom-6.0)  else begin $  ; Nominal frequencies in MHz
			fnom = h.freqs*h.freqs/80.5
			nnom = n_elements(fnom)
			dfnom = fnom(1:nnom-1)-fnom(0:nnom-2)
			m = median(dfnom)
   endelse
   for htimex=0,h.ntimes-1 do begin
     civil_time, year, month, day, daynum, hour, minute, second, h.times(htimex)
     timex = round( (60*(hour-sthr)+minute) / tdiff )		;get index of ionogram plot
      panelx = timex/times_per_panel            ; Panel holding that time slot

      ; Don't assign more than one ionogram to a time slot, or past last slot
      ;stop
      if (timex gt prev_timex) and (timex lt max_npanels*noofpanels*times_per_panel) then $
         begin
         ;stop
         ; Set up a new panel of ionograms if needed
         if panelx gt prev_panelx then begin
            if prev_panelx gt -1 then region_multi, /reset
            region_multi, /init, [ncols,nrows], pos=pos_regions(*,panelx)
            panel_stimex = timex
            endif

         ; Set-up axes location

         subplotx = timex mod times_per_panel
         subplot_col = subplotx/nrows           ; Left column is 0
         subplot_row = subplotx mod nrows       ; Top row is 0
         region_multi, [subplot_col, subplot_row]
         region_position, pos_in_region
         ;print,'axis plot'

         hrange = ends(h.heights)
         if keyword_set(heightrange) then hrange = heightrange
         if hrange(1) gt htlimit then hrange(1) = htlimit
         ;stop
         if not keyword_set(eldensityrange) then plot_oi, /nodata, /noerase, [0], [0], $
            xrange=mhzrange, xstyle=5, yrange=hrange, ystyle=5 else $
		plot, /nodata, /noerase, [0], [0], xrange = eldensityrange, xstyle = 5, $
		       yrange = hrange, ystyle = 5
         ; Plot ionogram, if any echoes (uniform steps in log(frequency)).

         if cadi.ndata gt 0 then begin          ; Any echoes in this hour?
            w = where(d.x.t eq htimex, wcount)  ; Any echoes for this time?
            if wcount gt 0 then begin
               d_ionogram = d(w)
			;if keyword_set(eldensityrange) then
;stop

			  ;try cleaning ionogram by removing isolated data points
			  ;print, 'no of data = ', n_elements(d_ionogram)
			   datanumber = 0
			   isolateddata = [0]
			   repeat begin
    		     heightsel = where ((d_ionogram.x.h eq d_ionogram(datanumber).x.h) or (d_ionogram.x.h eq d_ionogram(datanumber).x.h+1) or (d_ionogram.x.h eq d_ionogram(datanumber).x.h-1),nhgtsel)
;stop
			  if nhgtsel gt 1 then begin d_ionogramsel = d_ionogram(heightsel)
                 freqsel = where((d_ionogramsel.x.f eq d_ionogram(datanumber).x.f-1) or	(d_ionogramsel.x.f eq d_ionogram(datanumber).x.f+1),nhn)
			     if nhn gt 0 then isolateddata = [isolateddata,datanumber] else begin
                  freqsel = where (d_ionogram.x.f eq d_ionogram(datanumber).x.f,nfrsel)
                  if nfrsel gt 1 then d_ionogramsel = d_ionogram(freqsel)
                  hsel = where((d_ionogramsel.x.h eq d_ionogram(datanumber).x.h-1) or (d_ionogramsel.x.h eq d_ionogram(datanumber).x.h+1),nfn)
			      if nfn gt 0 then isolateddata = [isolateddata,datanumber]
			     endelse
			  endif
			     datanumber = datanumber + 1
			   endrep until datanumber eq wcount - 1
			   isolateddata = isolateddata(1:n_elements(isolateddata)-1)
			   d_ionogram = d_ionogram(isolateddata)
			  ;stop

              ; Find rectangle display area containing all the data points
               min_xf = min(d_ionogram.x.f, max=max_xf)
               min_xh = min(d_ionogram.x.h, max=max_xh)
               fdim = max_xf - min_xf + 1
               hdim = max_xh - min_xh + 1

               ; Create an image of this rectangle and plot it

               z = 0             ; Deallocate memory
               z = make_array( fdim, hdim, /byte, value=back_color)
              ;z = make_array( fdim, hdim, /int, value=back_color)
               if (size(z))(0) ne 2 then $
                  ; Ensure Z is 2D even if one or both dimensions are 1
                  z = reform( temporary(z), fdim, hdim )
                  if keyword_set(colour) then begin
			        for p = 0,n_elements(d_ionogram)-1 do begin
			          ;xpos = d_ionogram(p).x.f-min_xf
			          ;ypos = d_ionogram(p).x.h-min_xh
			          rept = fix(d_ionogram(p).IQ(0,0));-128
			          if (rept gt 127) then rept = rept - 256
			          impt = fix(d_ionogram(p).IQ(1,0));-128
			          if (impt gt 127) then impt = impt - 256
;print, 're,im = ', rept,impt
			          pow = rept^2 + impt^2
			          powdb = byte(7*10*alog10(pow))	;convert power to db and then scale to 255 for color
			          if (powdb gt 254) then powdb = 254
			     ;stop
			          ;if ((z(xpos,ypos) eq 255) or (powdb gt z(xpos,ypos))) then $
			           ;      z(xpos,ypos) = powdb
usersym, [-1.0,1.0,1.0,-1.0,-1.0],[-1.0,-1.0,1.0,1.0,-1.0],/fill

;need to check if intensity is maximum for that position
if (d_ionogram(p).x.f eq prevf) and (d_ionogram(p).x.h eq prevh) then begin
  if powdb gt prevpowdb then prevpowdb = powdb
endif else begin ;have maximum so plot
  symsize = float(powdb)/(255*times_per_panel)
  oplot, [h.freqs(d_ionogram(p).x.f)/1.0e6], [3*d_ionogram(p).x.h],psym = 8, color = 255 - prevpowdb, symsize = symsize + 0.2
  prevf = d_ionogram(p).x.f
  prevh = d_ionogram(p).x.h
  prevpowdb = powdb
endelse
			        endfor
                  endif else begin  ;end colour
                    z( d_ionogram.x.f-min_xf, d_ionogram.x.h-min_xh ) = fore_color
                    ;the plotting is done by the next statement
                    tvxy, z, fnom(min_xf:max_xf), h.heights(min_xh:max_xh), dx=m, dy=dh
                  endelse
               ;%% cadi_tvft, z, $
               ;%%    cadi.header.freqs(min_xf:max_xf) - candate, $
               ;%%    cadi.header.heights(min_xh:max_xh), $
               ;%%    dx=m, dy=dh, fore_color=fore_color, back_color=back_color

               endif ; wcount gt 0
            endif ; cadi.ndata gt 0

         ; Plot axes

         if subplot_row eq nrows-1 then begin
           if not keyword_set(eldensityrange) then begin
            xtitle = 'MHz'
            xtickname = def_ticknames
            endif else xtitle = 'el/m^3'
         endif else begin
            xtitle = ''
            xtickname = blank_ticknames
            endelse
         if (timex lt panel_stimex+nrows) then begin
            ytitle = 'km'
            ytickname = def_ticknames
         endif else begin
            ytitle = ' '
            ytickname = blank_ticknames
            endelse
         if not keyword_set(eldensityrange) then plot_oi, /nodata, /noerase, [0], [0], $
            xrange=10.0^!x.crange, xstyle=1, xtitle=xtitle, xtickname=xtickname, $
            yrange=!y.crange, ystyle=1, ytitle=ytitle, ytickname=ytickname, $
            yminor=2 else $
            plot, /nodata, /noerase, [0], [0], $
             xrange = eldensityrange, xstyle = 1, xtitle = xtitle, $
             yrange = !y.crange, ystyle = 1, ytitle = ytitle, ytickname = ytickname, $
             yminor = 2, xticks = 2, xcharsize = 0.5

         ; Write time on ionogram

         hhmm_str = string(hour, minute, format='(2i2.2)')
         if not keyword_set(eldensityrange) then xyouts, 1.3, hrange(1) - 50, '!c'+hhmm_str, $
            size=charsize else xyouts, (eldensityrange(0)+eldensityrange(1))/2.0, $
              heightrange(1) - 10, '!c' +hhmm_str, size = charsize/1.5

         prev_timex = timex
         prev_panelx = panelx
         endif ; timex gt prev_timex

      endfor ; timex
   end_hourx_loop:
  ;; endfor ; hourx


;if color then put on db scale
if keyword_set(colour) then begin
  yticklabels = ['0','6','12','18','24','30','36']
  plot, [0,1],[0,1],/nodata, yrange = [0,256], ystyle = 1, xrange = [0,1],$
     position = [0.95,0.18,0.98,0.85], xstyle = 4, yticks = 6, ytickname = yticklabels, title = 'dB'
  for c = 0,255 do oplot, [0,1],[c,c],color = 255 - c, thick = 3
endif



region_multi, /reset

!p.noerase = 0
!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
if not keyword_set(error_message) then error_message = ''
close, /all
region_multi, /clear
message, /inform, 'Aborted. '+error_message
xyouts, 0.3, 0.04, /norm, "Aborted ionogram summary plot for " + $
   datestr_in + ', ' + dir + '*.' + file_ext + $
   '!c' + error_message
return

end
