;+
; NAME:
;       CADI_SKYMAPS
; PURPOSE:
;       Plot CADI skymaps (maps of source locations coded by Doppler shift).
; CALLING SEQUENCE:
;       CADI_SKYMAPS, CADI_KVEC
;
;       Example: Plot all skymaps within time range TR.
;
;          IQ4MHZ = CADI_READ_DIR( $
;             DIR=DIR, TIMER=TR, FR=[2, 2], /INF, EXT='md1')
;          KVEC = CADI_XAP2KVEC( CADI_IQ2XAP( IQ4MHZ ))
;          CADI_SKYMAPS, KVEC
;
;       Example: Plot all skymaps with times of form hh:m4:50 and hh:m9:50, $
;             i.e. every 5 minutes.
;
;          IQ4MHZ = CADI_READ_DIR( $
;             DIR=DIR, TIMER=TR, FR=[2, 2], /INF, EXT='md1')
;          KVEC = CADI_XAP2KVEC( CADI_IQ2XAP( IQ4MHZ ))
;          TB = CADI_TBLOCKS( KVEC )
;          W = WHERE( ((TB.TIME+10) MOD 300) EQ 0)
;          CADI_SKYMAPS, KVEC, TIMEX=W, FSTEP=2, AXIS=2, NCOLS=6, NROWS=4
;
; INPUTS:
;       CADI_KVEC = Variable of CADI structure containing KVEC data.
; KEYWORD PARAMETERS:
;       FSTEP = Numeric scalar. Step in Doppler shift (Hz) between colors.
;          Default is 2.
;       AXISMAX = Numeric scalar. Extent of map axes in each direction from
;          zenith. Default is 1.
;       NOCOLOR: If present and nonzero, output is greyscale. Default is color.
;       SYMSIZE = Numeric scalar. Size of symbols used for sources.
;       SYMTHICK = Numeric scalar. Thickness of line used to draw source
;          symbols.
;       PUBLICATION: If present and nonzero, the username and date/time are
;          not written on the plot. Default is to write these on the plot.
;       NCOLS = Integer scalar. Number of columns of skymaps per page.
;       NROWS = Integer scalar. Number of rows of skymaps per page.
;       Pages are landscape and skymaps are square. So, for example, NCOLS=10
;          and NROWS=6 fills a page with 60 skymaps using most of the page.
;       TIMEX_LIST = integer vector. List of indices to select a subset of
;          times for plotting. These are indices into the sequence of times
;          for which there CADI_KVEC actually has sources, not into
;          CADI_KVEC.HEADER.TIMES. Thus after plotting (without TIMEX_LIST)
;          all skymaps for CADI_KVEC, a subset could be plotted with a
;          subsequent call to CADI_SKYMAPS using the numbers of the desired
;          subplots as elements of TIMEX_LIST.
;       PANEL_OFFSET = scalar integer. Number of subplot positions to skip
;          before plotting the first skymap.
;       POS_REGIONS = 4-element float vector. Limits [x0,y0,x1,y1] in
;          normalized coordinates of area of page in which all skymaps are to
;          be plotted.
;       TITLE = Scalar string to be used as the second line of the plot title.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 13 Nov 1994: Written by adapting CADI_FT_SKYMAPS.PRO.
;       GRANT, 28 Oct 1994: Added TITLE keyword.
;       GRANT, 4 Nov 1994: Changed default FSTEP to 2.
;       GRANT, 16 Jan 1995: Replaced keyword COLOR (with default value TRUE)
;          by keyword NOCOLOR (with default value FALSE) i.e. default is still
;          color.
;       JMacD, 26May 1995: Added keyword pcl to set greyscale for pcl output
;              (pcl greyscale is opposite of ps)
;		JWM april 2003 added dump of azimuth angles
;-
;
;       Example:
;          IQ4MHZ = CADI_READ_DIR( $
;             DIR=DIR, TIMER=TR, FR=[2, 2], /INF, EXT='md1')
;          KVEC = CADI_XAP2KVEC( CADI_IQ2XAP( IQ4MHZ ))
;          TB = CADI_TBLOCKS( KVEC )
;          W = WHERE( ((TB.TIME+10) MOD 300) EQ 0)
;          CADI_SKYMAPS, KVEC, TIMEX=W, FSTEP=2, AXIS=2, NCOLS=6, NROWS=4

pro cadi_skymaps, cadi_kvec, fstep=fstep, axismax=axismax, $
   nocolor=nocolor, symsize=symsize, $
   symthick=symthick, publication=publication, $
   ncols=ncols, nrows=nrows, $
   timex_list=timex_list, panel_offset=panel_offset, $
   pos_regions=pos_regions, title=title_in, $
   log=log, outpos=outpos, $
   pcl = pcl, overplot = overplot

;%% Keywords OUTPOS and LOG not documented for user, and not used by GRANT.

on_error, 0

yselected = [0]
if cadi_kvec.datatype ne 'KVEC' then $
   message, 'Argument does not have KVEC (k-vector of sources) data'
nsources = cadi_kvec.ndata
if nsources eq 0 then begin
   message, /inform, 'No sources in CADI_KVEC'
   return
   endif
header = cadi_kvec.header
sources_kvector = cadi_kvec.data

!p.background = 255b
!p.color = 0b
erase


;stop
if not keyword_set(ncols) then ncols = 4
if not keyword_set(nrows) then nrows = 3
; PANEL_OFFSET allows initial panels to be blank, e.g. to allow stacking
; of overhead transparencies from different sites.
if not keyword_set(panel_offset) then panel_offset = 0
npanels = nrows*ncols
psyms_nocolor = [8,1]   ; Minus (usersym), plus

if not keyword_set(pos_regions) then $
   pos_regions = [0.08,-1,0.85,0.95]
subtitle_ynorm = 0.05   ; Space for title of each subplot, norm y distance
pos_regions(1) = pos_regions(3) - $
   ((((pos_regions(2) - pos_regions(0)) / ncols * aspect(/device)) + $
    subtitle_ynorm) * nrows)
;stop
if pos_regions(1) lt 0.02 then $
   message, /inform, 'Too many rows (' + strim(nrows) + $
      ') for this number of columns (' + strim(ncols) + ')'

if not keyword_set(axismax) then $
      axismax = 1.0
if not keyword_set(fstep) then fstep = 2.0


;%% if n_elements(color) eq 0 then color = !true
if not keyword_set(nocolor) then nocolor = !false
if nocolor then $
   color = !false $
else $
   color = !true
if not keyword_set(symsize) then $
   symsize = 1 ;0.4
if not keyword_set(symthick) then $
   symthick = 1.0
if not keyword_set(log) then log = ''
axis_func_text = 'tan'
outpos = fltarr(4,ncols*nrows)  ; Space to output subplot positions

; Set up colors

if color then begin ;%%
  colors = [254,242, 228,220,167,112,96,80,70]
 ;;  tvlct_lj_posneg			;colors for a printer created by grant, see ifg_util
 ;;  colors = [3,4,5,6,14,8,9,10,11]              ; Cyans, grey, magentas
   endif $
else if not keyword_set(pcl) then begin
   colors = [0,11*16,255] ; For B/W Postscript. White=255, black=0 ;%% Was 11*16
   endif $
else begin
   colors = [255,11*16,0] ; for pcl color scale is opposite of postscript
   endelse
halfncolors = (n_elements(colors) - 1) / 2
centre_psym = 1 ; +
usersym_x = cos(circle_args(16)) ;%%
usersym_y = sin(circle_args(16)) ;%%

p_noerase = !p.noerase
!p.noerase = 1

; Adjust and replot loop

;%% repeat begin ; Adjust and replot
print, ' '
axis_max_found = 0.0
f_max_found = 0.0

; Set spatial scale

;%% axismax = enterdef('FLOAT', 'Max ' + axis_func_text + '(theta)', axismax)
xrange = [-axismax,axismax]
yrange = [-axismax,axismax]

; Set Doppler frequency scale

;%% fstep = enterdef('FLOAT', 'Frequency step', fstep)
levels=fstep * (findgen(2*halfncolors)-halfncolors+0.5)

; Hardcopy loop

;%%   repeat begin ; hardcopy

time_blocks = cadi_tblocks(sources_kvector, ntimes=ntimes)
if keyword_set(timex_list) then begin
   time_blocks = time_blocks(timex_list)
   ntimes = n_elements(timex_list)
   endif
nplots = 0      ; Counts pages (sheets)
for timex=0,ntimes-1 do begin

      s_1t = $
         sources_kvector(time_blocks(timex).startx : time_blocks(timex).endx)
      nsources = time_blocks(timex).endx - time_blocks(timex).startx + 1

      panelx = (timex + panel_offset) mod npanels ; Indexes panel location on page
      new_page = (timex eq 0) or (panelx eq 0)
      if new_page then begin

         ; Start new plot

         if nplots ne 0 then begin
            ; Finish old plot
            region_multi, /reset
            if not keyword_set(publication) then time
            ;dummy = enterdef('STRING', '<ENTER> to continue', '')
         endif
         nplots = nplots+1
         ;STOP
         if p_noerase eq 0 then erase

         minfreqx = min(sources_kvector.x.f, max=maxfreqx)
         if minfreqx eq maxfreqx then $
            freq_str = $
               strim(header.freqs(minfreqx)/1e6,format='(f6.3)') + ' MHz' $
         else $
            freq_str = 'Multi-freq'

         title='CADI skymaps    ' + header.site + '    ' + $
            strcantime(header.time,/date) + '   ' + freq_str ;%%
         if keyword_set(title_in) then $
            title = title + '!c' + title_in

         ;%% if keyword_set(hx_range) then $
         ;%%    title = title + '    ' + strim_float(header.heights(hx_range(0)))+ $
         ;%%       '-' + strim_float(header.heights(hx_range(1))) + ' km'
;stop
;if nplots eq 1 then $
         region_multi, /init, [ncols,nrows], pos=pos_regions, $
            charsize=2.25*pcharsize(), title=title
         ;%% outpos(0,mapx) = !p.region ;%% MAPX out of range after first page
         endif ; new_page       ;%% ; panelx eq 0

      ; Plot axes, etc.

      region_multi, panelx
      setmap, xrange=xrange, yrange=yrange, xlen=1.0, origin=[0,0]

      ; Align ends of color legend with top row of plots

      if new_page then $
         color_legend, /open, colors=colors, labels=strim_float(levels), $
            pos=[0.89,!p.position(1),0.94,!p.position(3)], title='  Hz', $
            borders=nocolor ;%% (color ne !true)

      ; Label only the lowest panel in the left column

      ;%% Modify to label a panel even if last row is not used.
      ;%% Also, check this works if PANELX_OFFSET NE 0.
      if ((panelx mod ncols) eq 0) and (npanels-panelx le ncols) then begin
         ticknames = strarr(30)
         case !p.font of
              -1: str_theta = '!4h!3'
               0: str_theta = '!9q!3'
            else: str_theta = 'theta'
            endcase
         xtitle = axis_func_text + ' ' + str_theta + '!IE!N'
         ytitle = axis_func_text + ' ' + str_theta + '!IN!N'
      endif else begin
         ticknames = replicate(' ',30)
         xtitle = ''
         ytitle = ''
         endelse

 if not (keyword_set(overplot)) or (keyword_set(overplot) and (nplots eq 1)) then begin
      plot, [0], [0], /nodata, $
         charsize=1.8, xcharsize=1.25, ycharsize=1.25, ticklen=0.04, $
         xrange=xrange, xstyle=1, xtickname=ticknames, xtitle=xtitle, $
         yrange=yrange, ystyle=1, ytickname=ticknames, ytitle=ytitle, $
         title = strcantime(header.times(s_1t(0).x.t),/time)
;stop
endif
         ;%% title = strcantime(time,/time) + '  ' + $
         ;%%    strim(header.freqs(freq_ptr)/1e6,format='(f6.3)') + '  ' + $
         ;%%    '   ' ; height
;%%      if panelx eq npanels-1 then $          ; Print units under last panel
;%%         xyouts, mean(xrange), yrange(0), '!c   UT      MHz    km', $
;%%            alignment=0.5

      ; Find and plot sources

      if nsources gt 0 then begin               ; If at least one source

         ; Normalize k-vectors

         x = s_1t.k(0) / s_1t.k(2)
         y = s_1t.k(1) / s_1t.k(2)

ysel = where(abs(x) lt 0.1, count)  ; select data near central y axis (north-south) of plot
if count gt 0 then yselected = [yselected,y[ysel]]

         ;%% Could more efficiently convert .X.D to COLORX_LIST directly,
         ;%% without going through DOPS. But need DOPS below to find MAX(ABS(DOPS)).
         ;%% Would have to convert LEVELS to equivalent HEADER.DOPS.
         dops = header.dops(s_1t.x.d)
         colorx_list = $
            bin( dops, [-1e30,levels,1e30], colors)
         ;%% colorx_list = bin( s(*).freq, [-1e30,levels,1e30], colors)

         ; Plot symbol interiors

         usersym, usersym_x, usersym_y, /fill
         plots, x, y, psym=8, symsize=symsize, color=colorx_list
           ;stop

         ; Plot symbol outlines

         ;%% if not color then begin
         if nocolor then begin
            usersym, usersym_x, usersym_y
            plots, x, y, psym=8, symsize=symsize
            ;stop
            endif

         axis_max_found = (axis_max_found > max(abs( [x,y] )))
         f_max_found = (f_max_found > max(abs(dops)))
         endif ; nsources gt 0

      ; Plot overhead point

      plots, [0], [0], psym=centre_psym

   endfor ; timex

; Finish last plot

region_multi, /reset
if not keyword_set(publication) then time

   ;%% endrep until hardcopy(/keep_ct)
print, 'Max |' + axis_func_text + '(theta)| = ' + strim_float(axis_max_found)
print, 'Max |f| = ' + strim_float(f_max_found)
;%% endrep until reply_char('Adjust scales and replot', 'yn', 'n') eq 'N'

;%% if nmaps gt 0 then outpos = outpos(*,0:((nmaps < npanels)-1) ) ;%% Tidy exceptions
!p.noerase = p_noerase

;set_plot, 'ps'
;device, filename = 'c:\plotfiles(temp)\hist.ps'
;!p.noerase = 0
;!P.POSITION = [0.1,0.1,0.9,0.9]
;HIST = histogram(yselected,binsize = 0.1); for later hour selecteed 0.3
;htotal = total(HIST)
;HIST = 100.0*float(HIST)/htotal
;bins = findgen(n_elements(hist))*0.1 + min(yselected)
;bins = (180.0/!pi)*atan(bins)
;plot, bins,hist,ytitle = '% reflectors', xtitle = 'azimuth angle', $
 ;    title = 'histogram distribution of reflectors along n-s axis of skymap (within 6 deg of axis)'
;device, /close

return
end
