;+
; NAME:
;	CADI_TBLOCKS
; PURPOSE:
;	Return indices to time blocks in a vector of CADI DATA structures.
;
; Advanced example:
;
; TFH is a CADI structure holding data from 18 to 24 UT.
; We want to create a new structure TFH20 with just the subset from 20 to 22 UT.
;
; tr20 = cantime(1993,12,2,20) + [0,2*3600L]	; Define timerange 20 to 22 UT.
; tb = cadi_tblocks(tfh)			; Get list of times in TFH.
; w = within( tb.time, tr20)			; Which TFH times are in range?
; tfh20 = $					; ENDS(W) points to first and
;  cadi_sub_struct( tfh, tr=tb(ends(w)).timex )	;  last relevant elements of TB.
; print, strcanrange( tfh20.header.times(tfh20.data.x.t) )	; Check
; This prints: 2-Dec-1993 20:01:08-21:59:57
;
; CALLING SEQUENCE:
;	CADI_TBLOCKS( ARG )
; INPUTS:
;	ARG =
;	   Either:
;	      Variable of CADI structure.
;	   Or:
;	      Vector of structures of CADI.DATA form.
;	      i.e. each element of ARG should have a field X, and possibly may
;	      have another field such as IQ, XAP or KVEC.
;	   Below, "SOURCES" refers to ARG.DATA in the first case, or ARG in
;	   the second.
; KEYWORD PARAMETERS:
;	NTIMES = scalar integer, returns number of elements in Result.
; OUTPUTS:
;	Result = Vector of structures.
;	   Result has one element for each time represented in SOURCES, as
;	   distinguished by the values of SOURCES.X.T.
;	   The fields of Result are:
;	      TIMEX  = Index into .HEADER.TIMES.
;	      TIME   = Element of .HEADER.TIMES, returned only if argument is a
;	               full CADI structure with a HEADER field.
;	      STARTX = Index of first element of SOURCES for this time.
;	      ENDX   = Index of last element of SOURCES for this time.
;	   All elements of SOURCES( Result(i).STARTX : Result(i).ENDX ).X.T
;	   have the same value, and no other elements of SOURCES.X.T have this
;	   value (assuming monotonicity).
; RESTRICTIONS:
;	The argument SOURCES must have a field X with subfield T.
;	The routine assumes these time indices are in order, that is
;	SOURCES.X.T is monotonic.
; PROCEDURE:
;	 Uses IDL's UNIQ function on SOURCES.X.T.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 14 Sep 1994: Written.
;	GRANT, 28 Oct 1994: Added TIME field to Result, and ability to handle
;	   both CADI structure and CADI.DATA vector.
;	GRANT, 10 Nov 1994: Added TIMEX field to Result.
;       GRANT, 8 Feb 1995: Prevented problem with UNIQ for exactly one source.
;-

function cadi_tblocks, arg, ntimes=ntimes

on_error, 0 ;2

timex_list_null = -1L

full_structure = in('HEADER', tag_names(arg))
if full_structure then $
   sources = arg.data $
else $
   sources = arg

if (size(sources)) (0) eq 0 then begin
   ntimes = 0
   return, timex_list_null
   endif

; Find number of unique times (NTIMES) used by SOURCES, and the
; indices of the first and last elements of SOURCES for each unique
; time (vectors STARTX_UNIQTIME and ENDX_UNIQTIME).
;
; Assume times are in order, so d.x.t monotonic. UNIQ will then return the
; index into SOURCES corresponding to the last element for each time.

endx_uniqtime = array(uniq(array( sources.x.t )))
ntimes = n_elements(endx_uniqtime)

; Create TIMEX_LIST, the vector of structures which will be Result.
;%% No! %%: Ensure the fields of Result have the same datatype as SOURCES.X.T.

;%% xt_example = sources(0).x.t	; To set datatype of Result fields
;%% timex_list = replicate( {time:-1L, startx:xt_example, endx:xt_example}, ntimes)
timex_list = replicate( {timex:-1L, time:-1L, startx:-1L, endx:-1L}, ntimes)

; Fill in the fields of Result.
if ntimes eq 1 then timex_list.endx = endx_uniqtime(0) else $
timex_list.endx = endx_uniqtime
timex_list(0).startx = 0
if ntimes gt 1 then $
  if ntimes eq 2 then  timex_list(1:*).startx = endx_uniqtime(0) + 1 $
   else timex_list(1:*).startx = endx_uniqtime(0:ntimes-2) + 1
timex_list.timex = sources(timex_list.startx).x.t
if full_structure then $
   timex_list(*).time = arg.header.times( timex_list.timex )

return, timex_list
end
