;+
; NAME:
;    CMEM
; PURPOSE:
;    Returns MEM spectrum evaluated at an arbitrary set of frequencies.
; CALLING SEQUENCE:
;    Result = CMEM(CX, ORDER)
; INPUTS:
;    CX = Complex time series. Converted to complex if not already.
;    ORDER = Scalar integer. Order of AR model = Number of Prediction Error 
;       Filter coefficients. ORDER = N_ELEMENTS(CX)/4 is conservative and a 
;       good rule of thumb.
; KEYWORD PARAMETERS:
;    F_IN = Input. Frequencies at which the PSD (spectrum) is to be evaluated,
;       scaled such that the sampling frequency is 1.0, i.e. the Nyquist
;       frequency is 0.5.
;       If a vector, the supplied frequencies are used.
;       If a scalar LE 1.1, the range 0.0 to F_IN is covered in 100 equal steps.
;       If a scalar GT 1.1, the range 0.0 to 0.5 is covered in F_IN equal steps.
;          In this case the FFT is used to efficiently evaluate the PSD.
;          For F_IN=10000, this is about 5 times faster than with F_IN specified
;	   as a vector.
;       If omitted, the range 0.0 to 0.5 is covered in 100 equal steps.
;    F_OUT = Output. The vector of frequencies at which the PSD was evaluated,
;       scaled as for F_IN.
;    A = Output. Complex vector of length ORDER+1. The AR filter parameter 
;       estimates A(0),...,A(ORDER). Note A(0)=1.0 always.
;    SIG2 = Output. Real scalar. Excitation white noise variance estimate.
;    POLES = Integer scalar. If present and non-zero the parameters of the
;            poles of the AR model are returned in FPOLES and/or DBPOLES.
;    FPOLES = FLOAT vector. Frequencies of poles of AR model. POLES must be set.
;    DBPOLES = FLOAT vector. Power in dB from poles of AR model. POLES must be
;            set. IFG is not sure if this interpretation of pole amplitudes is
;            correct.
; OUTPUTS:
;    Result = Real vector with length of F_OUT. PSD (spectrum) evaluated at
;       frequencies F_OUT.
; PROCEDURE:
;    This routine implements the Burg method for the estimation of the AR model
;    parameters, from which the spectrum is computed.
; VERIFICATION TEST CASE:
;    If the complex test data listed in Kay Appendix 1.A is input
;    as CX, and ORDER=10, then A and SIG2 should be
;       A(0) = ( 1.00000, 0.00000    )
;       A(1) = (-0.10964, 9.80833E-02)
;       A(2) = (-1.07989,-8.23723E-02)
;       A(3) = (-0.16670,-7.09936E-02)
;       A(4) = ( 0.86892, 0.14021    )
;       A(5) = (-0.60076,-3.46992E-02)
;       A(6) = ( 0.34997,-0.34456    )
;       A(7) = ( 0.59898, 0.15030    )
;       A(8) = (-0.20271, 0.34466    )    
;       A(9) = (-0.50447,-3.14139E-02)
;       A(10)= ( 0.37516,-0.18268    )
;       SIG2 = 0.16930
;    for a DEC VAX 11/780.
; MODIFICATION HISTORY:
;    Ian Grant, HIA/NRCC, 17 Dec 1991: Written in IDL2.
;       Adapted from subroutine BURG, Appendix 7D, in "Modern Spectral
;       Estimation: Theory and Application" by S.M. Kay, Prentice-Hall, 1988.
;    IFG, 22 Dec 1992: Added ON_ERROR.
;-

function cmem, cx, order, f_in=f_in, f_out=f_out, a=a, sig2=sig2, $
   poles=poles, fpoles=fpoles, dbpoles=dbpoles

on_error, 2
if not keyword_set(poles) then poles = !false
if not poles and ( (n_elements(fpoles) ne 0) or (n_elements(dbpoles) ne 0) ) $
   then message, 'Must set POLES to use FPOLES or DBPOLES'

x = complex(cx)
n = n_elements(x)
p = order
nfft = 0
case n_elements(f_in) of
   0: nfft = 200			; 0.0 to 0.5 in 100 steps
   1: if f_in le 1.1 then $
         f = 0.01*f_in*findgen(101) $	; 0.0 to f_in in 100 steps
      else $
         nfft = 2*round(f_in)		; Use FFT for 0.0 to 0.5 in F_IN steps
   else: f = f_in			; Use f_in vector supplied
   endcase
if nfft gt 0 then $
   f = 1.0/nfft*findgen(nfft/2+1)
f_out = f

rho = fltarr(p+1)
aa = complexarr(p+1,p+1)
efk1 = complexarr(n)
ebk1 = efk1
efk = efk1
ebk = efk1

;   Compute the estimate of the autocorrelation at lag zero (7.39 in Kay)

rho(0) = total(abs(x)^2) / n

;   Initialize the forward and backward prediction errors (7.39)

efk1 = x
ebk1 = x 

;   Begin recursion

for k=1,p do begin

   ;   Compute the reflection coefficient estimate (7.40)

   i = k+indgen(n-k)	; i = k,...,n-1
   aa(k,k) = -2.0 * $
      total(efk1(i)*conj(ebk1(i-1))) / total(abs(efk1(i))^2 + abs(ebk1(i-1))^2)

   ;   Update the prediction error power (7.40)

   rho(k) = (1.0-abs(aa(k,k))^2) * rho(k-1)

   if p eq 1 then goto, recursion_done

   ;   Update the prediction error filter coefficients (7.40)

   if k gt 1 then begin
      j = 1+indgen(k-1)	; j = 1,...,k-1
      aa(j,k) = aa(j,k-1) + aa(k,k)*conj(aa(k-j,k-1))
      endif ; k gt 1

   ;   Update the forward and backward prediction errors (7.41)

   i = k+1+indgen(n-k-1)	; i = k+1,...,n-1
   efk(i)    = efk1(i) + aa(k,k)*ebk1(i-1)
   ebk(i-1)  = ebk1(i-2) + conj(aa(k,k))*efk1(i-1)
   efk1(i)   = efk(i)
   ebk1(i-1) = ebk(i-1)

   endfor ; k

;   Find final values of the prediction error power,
;   which is the white noise variance estimate, and the
;   prediction coefficients, which are the AR filter
;   parameter estimates

recursion_done:
sig2 = rho(p)	; Noise variance estimate
a = aa(0:p,p)	; AR coefficients estimate
a(0) = 1.0

; Calculate spectrum.

if nfft eq 0 then begin
   ; Calcualte PSD separately for each frequency.
   ; The following is equivalent to equation (6.72) in Kay. Here the sum runs
   ; over k=0,...,p, and for k=0 the term is a(0)*exp(0)=1.
   minusj2pi = -complex(0.0,1.0)*2.0*!pi
   psd = f	; For size only
   for fx = 0,n_elements(f)-1 do $
      psd(fx) = sig2 / abs(total( a*exp(minusj2pi*findgen(p+1)*f(fx)) ))^2
endif else begin
   ; If the frequencies are regularly spaced from 0 to the Nyquist frequency,
   ; the FFT can be used to calculate the PSD efficiently. This idea is used by
   ; Kay's routine ARMAPSD, Appendix 5C.
   den = complexarr(nfft)
   den(0) = a
   den = nfft*(fft(den,-1))(0:nfft/2)	; Subscript DFT to get only freqs >= 0
   psd = sig2 / (float(den)^2 + imaginary(den)^2)
   endelse

; Calculate parameters of poles

if poles then begin
   zroots, a, z1
   z = 1.0/z1
   fpoles = atan(imaginary(z),float(z))/(2.0*!pi)
   dbpoles = db(1.0/(1.0-abs(z))^2)	;%% Not sure about this

   ; Return only non-negative frequencies, in descending order of power.

   w = where(fpoles gt -1e-6)
   fpoles = fpoles(w)
   dbpoles = dbpoles(w)
   sortx = reverse(sort(dbpoles))
   fpoles = fpoles(sortx)
   dbpoles = dbpoles(sortx)
   endif

return, psd
end
