;+
; NAME:
;	SUBRECT_WHERE
; PURPOSE:
;	Return 2D index limits of smallest subarray spanning nonzero elements.
;	For any 2D array Z, determine the min and max column (X) and row (Y)
;	indices of the minimum subarray which includes all of the nonzero
;	points of Z. Useful for trimming images which are blank at the edges, 
;	for plotting efficiency.
; CALLING SEQUENCE:
;	Result = SUBRECT_WHERE( Z )
; INPUTS:
;	Z = Two-dimensional array of any type.
; OUTPUTS:
;	Result = 4-element vector of long integers (x0, y0, x1, y1).
;	   Any element of Z not in the subarray (x0:x1, y0:y1) is zero, and 
;	   this is the smallest subarray of Z which includes all the nonzero
;	   elements of Z.
;	Result = -1 if all of Z is zero.
; PROCEDURE:
;	Determine the column indices of the first and last nonzero element in
;	each nonempty row.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 29 Oct 1993: Written.
;-

function subrect_where, z

on_error, 2
s = size(z)
if s(0) ne 2 then message, 'Argument is not two dimensional'
nrows = s(2)
row_limits = lonarr(2,nrows)

for row=0,nrows-1 do $
   ; Record column indices of first and last nonzero element in each row.
   ; Record [-1,-1] if row is empty (whole row is zero).
   row_limits(0,row) = ends(where(z(*,row)))

; Determine which rows are nonempty.

w = where(row_limits(0,*) ne -1, wcount)  ; Expect ROW_LIMITS(1,*) the same

if wcount eq 0 then $
   return, -1L $
else $
   ; [x0, y0, x1, y1 ]
   return, [ min(row_limits(0,w)), w(0), max(row_limits(1,w)), last(w) ]
end
