;+
; NAME:
;	TVLCT_LJ_IFG
; PURPOSE:
;	Loads 16 colours for the LJ-250 printer.
;
;	14 colours, plus black and white.
;	First 14 colours were selected from Table D-1 (pages D-9 and D-10), for 
;	Lightness 43-57 and Saturation 67-100. Then two of these were 
;	replaced with two light shades of magenta.
;	They include, in the following order, blue, cyan, green, yellow, red,
;	magenta.
; CALLING SEQUENCE:
;	TVLCT_LJ_IFG
; INPUTS:
;	None.
; SIDE EFFECTS:
;	Loads color tables.
; PROCEDURE:
;	Uses Table 7-6 in LJ-250 Programmer Reference Manual for RGB equivalents
;	of palette row/column numbers.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 14 May 1991, IDL2.
;	IFG, 22 May 1991: Removed a yellow-green (15/6) and an orange-red (10/2)
;	and added two pale magentas (6/9 and 6/10). This eliminates two regions 
;	of the sequence where the gradation was too fine, and shifts the
;	yellow/yellow-green boundary to the centre, so negative and positive
;	can be easily distinguished when zero is at the centre of the scale.
;	IFG, 21 Jan 1993: Added ON_ERROR.
;-

pro tvlct_lj_ifg

on_error, 2

; 14 maximum saturation colors:

; [0/1, 2/10, 0/9, 23/5, 22/7, 18/1, 15/6, 14/1, 13/1, 12/3, 11/8, 11/4,
; 10/2, 8/7, 6/7, 25/6]

;%% r = 2.55 * [0, 10,  4,  3,  2,   3,  5, 12, 30, 89, 72, 62, 57, 53, 53, 100]
;%% g = 2.55 * [0,  4,  4, 10, 22,  26, 31, 38, 53, 83, 41, 21, 12,  8,  5, 100]
;%% b = 2.55 * [0, 27, 29, 46, 64,  22, 21, 18, 16, 13, 13, 13, 13, 14, 25, 100]


; Deleted a yellow-green and an orange-red, added two light magentas:

; [0/1, 2/10, 0/9, 23/5, 22/7, 18/1, 14/1, 13/1, 12/3, 11/8, 11/4,
; 8/7, 6/7, 6/9, 6/10, 25/6]

r = 2.55 * [0, 10,  4,  3,  2,   3, 12, 30, 89, 72, 62, 53, 53, 65, 76, 100]
g = 2.55 * [0,  4,  4, 10, 22,  26, 38, 53, 83, 41, 21,  8,  5, 19, 45, 100]
b = 2.55 * [0, 27, 29, 46, 64,  22, 18, 16, 13, 13, 13, 14, 25, 43, 62, 100]

tvlct, r,g,b
return
end
;+
; NAME:
;	TVLCT_LJ_POSNEG
; PURPOSE:
;	Loads 16 colours for the LJ-250 printer, to suit data with both signs.
;	Colors are: 3 blacks, 4 blues (dark to light), white, 5 warms
;	(yellow - orange - red), white, grey, white.
; CALLING SEQUENCE:
;	TVLCT_LJ_POSNEG
; INPUTS:
;	None.
; SIDE EFFECTS:
;	Loads color tables.
; PROCEDURE:
;	Uses Table 7-6 in LJ-250 Programmer Reference Manual for RGB equivalents
;	of palette row/column numbers.
; MODIFICATION HISTORY:
;	Ian Grant, UWO, 24 Sep 1993: Written.
;	GRANT, 25 Oct 1994: Changed hues to those selected by Claire Aligne to
;	   suit Don Moorcroft's color blindness.
;-

pro tvlct_lj_posneg

on_error, 2

; -----------------------------------------------------------------------------

; Hues developed by Ian Grant.

; Black, blue, blue-cyan, 4 cyans, white, 4 magentas, magenta-red, red,
; grey, white

; [0/1, 0/9, 24/4, 22/7, 22/8, 22/9, 22/10, 25/6, 6/10, 6/9, 6/8, 6/7, 8/5,
; 8/7, 25/5, 25/6]

; r = 2.55 * [0,  4,  4,  2,  3,  12, 38, 100, 76, 65, 58, 53, 53, 53, 43, 100]
; g = 2.55 * [0,  4,  7, 22, 27,  39, 58, 100, 45, 19,  7,  5,  7,  8, 43, 100]
; b = 2.55 * [0, 29, 36, 64, 68,  73, 78, 100, 62, 43, 31, 25, 16, 14, 45, 100]

; -----------------------------------------------------------------------------

; Hues taken from Claire Aligne's PLOT_COLORS_RL_VEL.PRO.
; These suit Don Moorcroft's red-green color blindness.

; 3 blacks, 4 blues (dark to light), white, 5 warms (yellow - orange - red),
; white, grey, white

; [0/1, 0/1, 0/1, 0/9, 23/5, 22/7, 22/9, 25/6, 12/6, 12/3, 11/8, 11/3, 8/7,
; 25/6, 25/5, 25/6]

r = 2.55 * [ 0, 0, 0,  4,  3,  0, 40, 100, 100, 90, 70, 70, 50, 100, 43, 100 ]
g = 2.55 * [ 0, 0, 0,  4, 10, 80, 90, 100,  90, 90, 40, 30, 10, 100, 43, 100 ]
b = 2.55 * [ 0, 0, 0, 30, 45, 80, 90, 100,  60, 15, 15, 15, 15, 100, 45, 100 ]

tvlct, r,g,b
return
end
