;+
; NAME:
;       HARDCOPY
; PURPOSE:
;       Offer hardcopy of graphics.
; CALLING SEQUENCE:
;       REPEAT BEGIN
;          Plotting code goes here.
;          It is executed on the first time through with graphics going to the
;          screen.
;          If the response to "Hardcopy wanted?" below is not 'n' or 'N', then
;          this code is executed a second time with graphics sent to a plot
;          file.
;          ENDREP UNTIL HARDCOPY()      ; Respond with Y or N
; INPUTS:
;       None.
; KEYWORD PARAMETERS:
;       OFFER: If present and non-zero, then hardcopy is offered (default),
;          If present and zero, then hardcopy is not offered.
;       FILENAME: If present and non-zero, it is used for the name of the
;          plotfile. Default is IDL.PS or IDL.LJ depending on the device.
;       KEEP_CT: If present and non-zero then this routine will use the
;          existing color table. The default is for this routine to set the
;          colour table according to the device.
;       PS: If present and non-zero, monochrome PostScript output is selected.
;       LJ: If present and non-zero, LJ-250 output is selected.
;       PCL,PCN: If present and non-zero, PCL command laguage output is selected.
; OUTPUTS:
;       HARDCOPY() returns true if and only if hardcopying is off after return.
; COMMON BLOCKS:
;       cmn_hardcopy
; RESTRICTIONS:
;       It is intended that HARDCOPY() only be used as the condition in a loop
;       as above under Calling Sequence.
;          Currently set up for hardcopy devices PostScript monochrome and
;       LJ-250 printer.
; PROCEDURE:
;       Uses common to save current state: hardcopying on/off state, device
;       characteristics, etc.
;       Each call toggles hardcopying on/off.
;       If hardcopy is off before call, current device characteristics are
;       saved, user is prompted to select a hardcopy device, plotfile is opened.
;       If hardcopy is on before call, plotfile is closed, initial device is
;       restored.
; MODIFICATION HISTORY:
;       Ian Grant, HIA/NRCC, 1991, IDL2. Written by adapting HARDCOPY_WANTED.
;       IFG, 15 Apr 1992: Removed check for X windows.
;       IFG, 22 Dec 1992: Added ON_ERROR.
;       IFG,  1 Feb 1993: Removed reference to user system variable !H.
;       IFG, 31 Mar 1993: Added Options choice, and keyword KEEP_CT.
;       IFG, 29 Apr 1993: For KEEP_CT true, now get color table before
;          changing device and set after.
;       IFG, 27 May 1993: Changed LJ sizes and offsets slightly so the plot
;          can go on the first sheet in the printer and not cross perforation.
;       IFG, 22 Jun 1993: Added keywords, PS, LJ. Passed from PROHARDCOPY to
;          allow hardcopy to be turned on and off without interactive input.
;       IFG, 26 jan 1995: Added keyword FILENAME. Added documentation for
;          KEEP_CT, PS, LJ keywords, which were implemented but undocumented.
;       JWM, 31 mar 1995: Added keyword, PCL
;       JWM, 4 Nov 1995: Added keyword PCN to produce narrow potrait orient plots.
;		JWM 24 jan 2002: added opimize=1 keyword for laserjet 5l printer
;		JWM 15 May 2003: added jpeg option
;-

function hardcopy, offer=offer, keep_ct=keep_ct, ps=ps, lj=lj, pcl = pcl, pcn = pcn, jpeg = jpeg, filename=filename

; Save between calls.
common cmn_hardcopy, hard$off, hard$filename, hard$dname_save, hard$device, $
   hard$landscape
on_error, 0
if n_elements(hard$off) eq 0 then hard$off = !true ; First call
if n_elements(offer) eq 0 then offer = !true    ; Default is to offer hardcopy
if n_elements(keep_ct) eq 0 then keep_ct = !false ; Default is set color table
want_stop = !false

if hard$off then begin
 if offer then begin

   if howmany([ keyword_set(ps), keyword_set(lj) , keyword_set(pcl) ,keyword_set(pcn), keyword_set(jpeg)]) gt 1 then $
      message, 'Must set at most one of PS, LJ, PCL, PCN, JPEG'
   if keyword_set(ps) then $
      hard_option = 'P' $
   else if keyword_set(lj) then $
      hard_option = 'L' $
   else if keyword_set(pcl) then $
      hard_option = 'C' $
   else if keyword_set(pcn) then $
      hard_option = 'M' $
   else if keyword_set(jpeg) then $
      hard_option = 'J' $
   else repeat begin ; until hard_option ne 'O'
      if keep_ct then $
         message, /inform, 'Default is to keep current color table'
      hard_option = reply_char( $
         'Hardcopy on [P]ostscript, [L]J-250, p[C]l, [O]ptions, or [N]one','plcon','n')
      if hard_option eq 'O' then begin

         ; Request and set options

         keep_ct = (reply_char( $
            'Use [C]urrent color table, or [N]ot', 'cn', 'n') eq 'C')
         want_stop = (reply_char( $
            '[S]top after DEVICE statement, or [N]ot', 'sn', 'n') eq 'S')
         endif ; hard_option eq 'O'
      endrep until hard_option ne 'O'

   if hard_option ne 'N' then begin

      ; Start plot

      hard$dname_save = !d.name
      hard$landscape = (!d.x_size ge !d.y_size)
      if keep_ct then $
         tvlct, /get, r, g, b

      case hard_option of
         'P': hard$device = 'PS'
         'L': hard$device = 'LJ'
         'C': hard$device = 'PCL'
         'M': hard$device = 'PCN'
         'J': hard$device = 'JPEG'
         endcase
       ;stop
      case hard$device of
         'PS': begin            ; Start PostScript plot
            if keyword_set(filename) then $
               hard$filename = filename $
            else $
               hard$filename = 'idl.ps'
            set_plot, 'ps'
            if hard$landscape then $
               device, /landscape, filename=hard$filename, $
                  xsize=25.0, ysize=20.0, xoffset=0.7, yoffset=26.5 $
            else $
               device, /portrait, filename=hard$filename, $
                  xsize=20.0, ysize=25.0, xoffset=0.7, yoffset=1.5
            end

        'LJ': begin            ; Start LJ plot
            if keyword_set(filename) then $
               hard$filename = filename $
            else $
               hard$filename = 'idl.lj'
            set_plot, 'lj'
            if hard$landscape then $
               device, resolution=90, depth=4, filename=hard$filename, $
                  /landscape, $ ; LJ offset units seem to be 0.25 cm = 0.1 inch
                  xsize=24.0, ysize=19.2, xoffset=1.0, yoffset=0.0 $
                  ;%% Following needs perforation just above print head
                  ;%% xsize=25.0, ysize=20.0, xoffset=1.0, yoffset=3.0 $
            else $
               device, resolution=90, depth=4, filename=hard$filename, $
                  /portrait, $
                  xsize=19.2, ysize=24.0, xoffset=0.8, yoffset=0.0
                  ;%% Following needs perforation just above print head
                  ;%% xsize=20.0, ysize=25.0, xoffset=0.8, yoffset=1.5
            if not keep_ct then tvlct_lj_ifg
            end

        'PCL': begin           ; Start PrinterCommandLanguage plot
            if keyword_set(filename) then $
               hard$filename = filename $
            else $
               hard$filename = 'idl.pcl'
            set_plot, 'pcl'
            if hard$landscape then $
               device, /landscape, filename=hard$filename, $
                  xsize=25.0, ysize=20.0, xoffset=0, yoffset=1.5, optimize = 1 $
            else $
               device, /portrait, filename=hard$filename, $
                  xsize=20.0, ysize=25.0, xoffset=0.7, yoffset=1.5
            end

         'PCN': begin           ; Start narrow PrinterCommandLanguage plot
            if keyword_set(filename) then $
               hard$filename = filename $
            else $
               hard$filename = 'idl.pcl'
            set_plot, 'pcl'
            device, /landscape, filename=hard$filename, $
                  xsize=25.0, ysize=9.0, xoffset=10, yoffset=1.5
           end

         'JPEG': begin            ; Start jpeg plot
            if keyword_set(filename) then $
               hard$filename = filename $
            else $
               hard$filename = 'idl.jpeg'
            set_plot, 'jpeg'
            if hard$landscape then $
               device, /landscape, filename=hard$filename, $
                  xsize=25.0, ysize=20.0, xoffset=0.7, yoffset=26.5 $
            else $
               device, /portrait, filename=hard$filename, $
                  xsize=20.0, ysize=25.0, xoffset=0.7, yoffset=1.5
            end

      endcase

      if keep_ct then $
         tvlct, r, g, b
      if want_stop then $
         stop, 'HARDCOPY: Stopped after DEVICE & TVLCT. Enter .CON to continue'
      hard$off = !false
      endif     ; hard_option ne 'N'
   endif        ; offer
 endif $        ; hard$off
else begin      ; hard$off is false

   ; Finish plot

   case hard$device of
      'PS': begin               ; Finish PostScript plot
         device, /close
         end
      'LJ': begin               ; Finish LJ plot
         device, /close
         end
      'PCL': begin              ; Finish PrinterCommandLanguage plot
         device, /close
         end
      'PCN': begin              ; Finish PrinterCommandLanguage plot
         device, /close
         end
       'JPEG': begin
         device, /close
         end
      endcase

   ; Restore screen plotting environment

   set_plot, hard$dname_save
   if !d.name eq 'X' then begin
      device, retain=2
      endif
   if not hard$landscape then $
      portrait

   hard$off = !true
   message, /inform, 'Plot file is ' + strupcase(hard$filename)
   endelse      ; hard$off is false

return, hard$off
end
